/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;

import houseexample.House;
import houseexample.Man;

import java.util.Iterator;
import javax.swing.JTextArea;
import javax.xml.parsers.ParserConfigurationException;
import junit.framework.TestCase;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentFactory;
import org.objectweb.kilim.model.ComponentInterface;
import org.objectweb.kilim.model.ComponentSlot;
import org.objectweb.kilim.model.RtComponent;
import org.objectweb.kilim.model.RtComponentElement;
import org.objectweb.kilim.model.RtSingleValuePort;
import org.objectweb.kilim.model.mapping.DefaultMappingContext;
import org.objectweb.kilim.model.mapping.NullMapper;
import org.objectweb.kilim.model.mapping.JavaRuntimeMapper;
import org.objectweb.kilim.model.mapping.MappingContext;
import org.objectweb.kilim.model.mapping.TextAreaMapper;
import org.objectweb.kilim.model.mapping.TreeModelMapper;
import org.objectweb.kilim.model.instanciation.DefaultInstanciationStrategy;
import org.objectweb.kilim.model.instanciation.InstanciationStrategy;
import org.objectweb.kilim.model.instanciation.NullInstanciationMger;
import org.objectweb.kilim.tools.KilimComponentViewer;
import org.objectweb.kilim.tools.KilimTemplateViewer;
import org.objectweb.kilim.tools.KilimTraceTreeModel;
import org.objectweb.kilim.tools.KilimTraceTreeViewer;

import org.xml.sax.SAXException;

/**
 * @author delpiano
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class CurrentTest extends TestCase {

	private ResourceRepository rep;
	private ResourceRepository rep1;

	/**
	 * Constructor for TemplateParserTest.
	 * @param arg0 for junit
	 * @throws SAXException should not happen
	 * @throws ParserConfigurationException should not happen
	 */
	public CurrentTest(String arg0) throws SAXException, ParserConfigurationException {
		super(arg0);
		TemplateDescriptionParser parser = new TemplateDescriptionParser(true);
		rep = new ResourceRepository(parser);
		rep.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
		rep1 = new ResourceRepository1(parser);
		rep1.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
	}
	
	public void testForkOfPlugged3() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Plugging Plug into Container with NAry Slot");		
			Component compo = ComponentFactory.newComponent(ts0);
			Component compo1 = compo.getSubComponent("Container");
			Component compo2 = compo.getSubComponent("Plug");
			System.out.println("RESULT " + compo2.getQualifiedName() + " de CONTENEUR " + compo2.getContainingComponent());
			Iterator iter = compo.getSubComponents();
			
			System.out.println("========================================================");
			while (iter.hasNext()) {
				Component tCompo = (Component) iter.next();
				System.out.println("sous composant " + tCompo.toString() + " :: " + tCompo.getContainingComponent());
			}
			Component compo3 = compo2.fork();
			System.out.println("RESULT " + compo3.getQualifiedName() + " de CONTENEUR " + compo3.getContainingComponent());

			System.out.println("========================================================");
			iter = compo.getSubComponents();
			while (iter.hasNext()) {
				Component tCompo = (Component) iter.next();
				System.out.println("sous composant " + tCompo.toString() + " :: " + tCompo.getContainingComponent());
			}
			Component compo4 = compo3.fork();
			System.out.println("========================================================");
			iter = compo.getSubComponents();
			while (iter.hasNext()) {
				Component tCompo = (Component) iter.next();
				System.out.println("sous composant " + tCompo.toString() + " :: " + tCompo.getContainingComponent());
			}
			
			Component compo5 = compo4.fork();
			System.out.println("========================================================");
			iter = compo.getSubComponents();
			while (iter.hasNext()) {
				Component tCompo = (Component) iter.next();
				System.out.println("sous composant " + tCompo.toString() + " :: " + tCompo.getContainingComponent());
			}
			
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "42424242-is--is--is--is-the answerthe answerthe answerthe answer");
			//correct result when using the preplug implementation			
			//assertEquals(rslt.toString() , "42-is-the answer42-is-the answer42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
}
