package org.objectweb.kilim.repository;

import houseexample.House;
import houseexample.Man;

import javax.swing.JFrame;
import javax.xml.parsers.ParserConfigurationException;
import junit.framework.TestCase;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.description.Slot;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentFactory;
import org.objectweb.kilim.model.ComponentInterface;
import org.objectweb.kilim.model.ComponentProperty;
import org.objectweb.kilim.model.ComponentSlot;
import org.objectweb.kilim.model.RtSingleValuePort;
import org.objectweb.kilim.model.mapping.DefaultMappingContext;
import org.objectweb.kilim.model.mapping.JavaRuntimeMapper;
import org.objectweb.kilim.model.mapping.TextAreaMapper;
import org.objectweb.kilim.model.mapping.TreeModelMapper;

import org.objectweb.kilim.model.instanciation.DefaultInstanciationStrategy;
import org.objectweb.kilim.model.instanciation.InstanciationStrategy;

import org.objectweb.kilim.tools.KilimComponentViewer;
import org.objectweb.kilim.tools.KilimTemplateViewer;
import org.objectweb.kilim.tools.KilimTraceTreeModel;
import org.objectweb.kilim.tools.KilimTraceTreeViewer;

import org.xml.sax.SAXException;

/**
 * @author delpiano
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class DynamicityTests extends TestCase {

	private ResourceRepository rep;
	private ResourceRepository rep1;

	public DynamicityTests(String arg0) throws SAXException, ParserConfigurationException {
		super(arg0);
		
		TemplateDescriptionParser parser = new TemplateDescriptionParser(true);
		rep = new ResourceRepository(parser);
		rep.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
		rep1 = new ResourceRepository1(parser);
		rep1.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
	}
	
	public void testManualPlug() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("dynamicity/Real House");
			Component house_component = ComponentFactory.newComponent(ts0);
			ComponentInterface interf1 = house_component.getInterface("house");


			TemplateDescription ts1 = rep.getTemplateDescription("dynamicity/Real Man");
			Component man_component = ComponentFactory.newComponent(ts1);
			ComponentInterface interf2 = man_component.getInterface("man");
			
			House house;
			house = (House) interf1.getValue();
			assertEquals(((RtSingleValuePort) interf1).hasValue(), true);
			assertEquals(house.readBoard() , "");
			
			Man man ;
			man = (Man) interf2.getValue();
			assertEquals(((RtSingleValuePort) interf2).hasValue(), true);
			assertEquals(man.whereILive() , null);
			
			ComponentSlot vstr_slt = house_component.getSlot("visitor");
			
			vstr_slt.plug(man_component);
			
			//next line is useless
			//man = (Man) interf2.getValue();
			assertEquals(man.whereILive() , house);
			
			//next line is useless					
			//house = (House) interf1.getValue();
			assertEquals(house.getInhibitants().next() , man);
			assertEquals(house.readBoard() , "It's me Toto the heroe, I'm home !");
		
			vstr_slt.unplug(man_component);			
			assertEquals(house.readBoard() , "It's me Toto the heroe, I'm home !Bye folks, Toto the heroe is leaving !");
			assertEquals(man.whereILive() , null);
						
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}
	
	public void testStaticPlug() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("dynamicity/SimpleAssembly");
			
			InstanciationStrategy strgy = new DefaultInstanciationStrategy();
			TextAreaMapper textMapper = new TextAreaMapper(new JavaRuntimeMapper());
			TreeModelMapper treeMapper = new TreeModelMapper(textMapper);			
			KilimTraceTreeModel treeModel = treeMapper.getTraceTree();
			Component assembly = ComponentFactory.newComponent(ts0);
			
			ComponentInterface interf1 = assembly.getInterface("house");
			ComponentInterface interf2 = assembly.getInterface("man");
						
			House house = (House) interf1.getValue();
			Man man = (Man) interf2.getValue();
			
			assertEquals(man.whereILive() , house);
			assertEquals(house.getInhibitants().hasNext() , true);
			assertEquals(house.getInhibitants().next() , man);
			assertEquals(house.readBoard() , "It's me Toto the heroe, I'm home !");
			
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}
		
	public void testChangePropertyValue() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {	
			TemplateDescription ts0 = rep.getTemplateDescription("dynamicity/DynamicProperty");
			Component frame_component = ComponentFactory.newComponent(ts0);
			ComponentInterface wndw = frame_component.getInterface("window");
			JFrame frm = (JFrame) wndw.getValue();
			
			assertEquals(frm.getWidth() , 200);
			assertEquals(frm.getHeight() , 200);
			
			ComponentProperty prprt = (ComponentProperty) frame_component.getInterface("x");
			prprt.setValue(new Integer(500));

			prprt = (ComponentProperty) frame_component.getInterface("y");
			prprt.setValue(new Integer(500));
			wndw.update();
											
			assertEquals(frm.getWidth() , 500);
			assertEquals(frm.getHeight() , 500);
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}
}
