/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;

import java.util.Iterator;
import java.util.LinkedHashMap;

import javax.xml.parsers.ParserConfigurationException;
import junit.framework.TestCase;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.KilimConfiguration;

import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentFactory;
import org.objectweb.kilim.model.ComponentInterface;
import org.objectweb.kilim.model.RtExternalValue;

import org.objectweb.kilim.model.instanciation.DefaultInstanciationStrategy;
import org.objectweb.kilim.model.instanciation.InstanciationStrategy;

import org.objectweb.kilim.model.mapping.DefaultMappingContext;
import org.objectweb.kilim.model.mapping.DefaultMappingStrategy;
import org.objectweb.kilim.model.mapping.JavaRuntimeMapper;
import org.objectweb.kilim.model.mapping.Mapper;
import org.objectweb.kilim.model.mapping.MappingContext;
import org.objectweb.kilim.model.mapping.MappingStrategy;
import org.objectweb.kilim.model.mapping.NullMapper;
import org.objectweb.kilim.model.mapping.TextAreaMapper;
import org.objectweb.kilim.model.mapping.TreeModelMapper;

import org.objectweb.kilim.model.services.Annotations;
import org.objectweb.kilim.model.services.ExternalValueReferences;

import org.objectweb.kilim.tools.KilimComponentViewer;
import org.objectweb.kilim.tools.KilimTemplateViewer;
import org.objectweb.kilim.tools.KilimTraceTreeModel;
import org.objectweb.kilim.tools.KilimTraceTreeViewer;

import org.xml.sax.SAXException;

/**
 * @author delpiano
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class MonoFeatureTests extends TestCase {
	
	private ResourceRepository rep;
	private ResourceRepository rep1;
	
	/**
	 * Constructor for TemplateParserTest.
	 * @param arg0 for junit
	 * @throws SAXException should not happen
	 * @throws ParserConfigurationException should not happen
	 */
	public MonoFeatureTests(String arg0) throws SAXException, ParserConfigurationException {
		super(arg0);
		TemplateDescriptionParser parser = new TemplateDescriptionParser(true);
		rep = new ResourceRepository(parser);
		rep.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
		rep1 = new ResourceRepository1(parser);
		rep1.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
	}
	
	/**
	 * Teste les tableaux de properties, nommes et non nommes.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testPropertiesArrays() throws ResourceNotFoundException, SAXException, ParserConfigurationException {	
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("writers/ArrayDummyExample");		
			//KilimTemplateViewer.viewTemplate(ts0);
			Component compo = ComponentFactory.newComponent(ts0);			
			ComponentInterface interf = compo.getInterface("array");
			interf.getValue();
			interf = compo.getInterface("array2");
			interf.getValue();
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}	
	}
	
	/**
	 * Teste le mcanisme de surcharge des valeurs de properties par des fichiers .prop.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 */
	public void testPropertiesOverloading() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/two");		
			Component compo = ComponentFactory.newComponent(ts0);	
					
			ComponentInterface interf = compo.getInterface("string port");
			Object res = interf.getValue();
			assertEquals(res.toString() , "Ceci est un test russi");
			
			interf = compo.getInterface("integer port");
			res = interf.getValue();

			assertEquals(((Integer) res).intValue() , 42);

			interf = compo.getInterface("long port");
			res = interf.getValue();

			assertEquals(((Long) res).longValue() , 42);
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}	
	}
	
	/**
	 * Teste l'ordre dans lequel les triggers sont excuts.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 */
	public void testTriggersOrder() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep1.getTemplateDescription("test/B states");		
			Component compo = ComponentFactory.newComponent(ts0);			
			ComponentInterface interf = compo.getInterface("initial");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals("123", rslt.toString());
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}	
	}
		
	/**
	 * Teste les bindings de property  interface, de provider  interface.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 */
	public void testPropertiesInterfaceAndProvidersBindings() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Plugging Plug into Container");		
			Component compo = ComponentFactory.newComponent(ts0);			
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}	
	}
	
	/**
	 * Teste la surcharge d'instances par super.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 */
	public void testSuperAndSuperSuper() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/PLentyOfInstances A");
			TemplateDescription ts1 = rep1.getTemplateDescription("test/PLenty of Instances B");
			Component compo = ComponentFactory.newComponent(ts0);			
			ComponentInterface interf = compo.getInterface("a of level 1");
			String rslt = (String) interf.getValue();
			assertEquals(rslt , "Valeur de a modifie");
			
			compo = ComponentFactory.newComponent(ts1);			
			interf = compo.getInterface("a of level 2");
			rslt = (String) interf.getValue();
			assertEquals(rslt , "Valeur de a modifie");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}	
	}

	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testExternalReference() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Container for Externals");		
			//Annotations xServ = new Annotations();
			ExternalValueReferences xRefs = new ExternalValueReferences();
			xRefs.addExternalValueReference("/extern/b", new String("-is-"));
			xRefs.addExternalValueReference("/extern/c", new String("" + 42));
			xRefs.addExternalValueReference("/extern/a", new String("the answer"));
			//xServ.addAnnotation("external references",  xRefs);
			//InstanciationStrategy strgy = new DefaultInstanciationStrategy();
			//xServ.addAnnotation("instanciation strategy", strgy);
			KilimConfiguration.setExternalReferences(xRefs);
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt_bff = (StringBuffer) interf.getValue();	
			
			String rslt = rslt_bff.toString();			
			assertEquals(rslt, "the answer-is-42");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Teste si le binding multiple lve bien une exception.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 */
	public void testMultipleBindingsDetection() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		boolean exception_thrown = false;
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Multiple Bindings Subtemplate");
			Component compo = ComponentFactory.newComponent(ts0);			
		} catch (KilimException e) { 
			e.printStackTrace();
			exception_thrown = true;
		}	 finally {
			assertEquals (exception_thrown , false);
		}
	}

	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testMultipleTypesOfOverloading() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		TemplateDescription ts0 = rep1.getTemplateDescription("test/Overloading Things");
		Component compo = ComponentFactory.newComponent(ts0);	
		assertEquals(compo.getInterface("a").getValue()	, "a modified");	
		assertEquals(compo.getInterface("b").getValue()	, "b modified");
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testOrderOnCompositionOfMultiplePort() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Plugging Plug into Container with NAry Slot");		
			Component compo = ComponentFactory.newComponent(ts0);			
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "4242-is--is-the answerthe answer");
			//correct result when using the preplug implementation
			//assertEquals(rslt.toString() , "42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testDynamicPlugOnNAryPort() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Container with NAry Slot");
			Component compo = ComponentFactory.newComponent(ts0);
			TemplateDescription ts1 = rep1.getTemplateDescription("test/Plug");		
			Component compo1 = ComponentFactory.newComponent(ts1);
			Component compo2 = ComponentFactory.newComponent(ts1);
			compo.plug("slot", compo1);
			compo.plug("slot", compo2);
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "4242-is--is-the answerthe answer");
			//correct result when using the preplug implementation
			//assertEquals(rslt.toString() , "42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testDynamicPlug() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Container");		
			Component compo = ComponentFactory.newComponent(ts0);
			TemplateDescription ts1 = rep1.getTemplateDescription("test/Plug");		
			Component compo1 = ComponentFactory.newComponent(ts1);
			compo.plug("slot", compo1);
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testSimpleFork1() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Container");		
			Component compo = ComponentFactory.newComponent(ts0);
			TemplateDescription ts1 = rep1.getTemplateDescription("test/Plug");		
			Component compo1 = ComponentFactory.newComponent(ts1);
			Component compo2 = compo1.fork();
			compo.plug("slot", compo2);
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testSimpleFork1A() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Container with Plug");		
			Component compo = ComponentFactory.newComponent(ts0);	
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "42-is-the answer");
			Component compo1 = compo.getSubComponent("Plug");
			Component compo2 = compo1.fork();
			interf.update();
			rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testSimpleFork2() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Container with NAry Slot");		
			Component compo = ComponentFactory.newComponent(ts0);
			TemplateDescription ts1 = rep1.getTemplateDescription("test/Plug");		
			Component compo1 = ComponentFactory.newComponent(ts1);
			Component compo2 = compo1.fork();
			Component compo3 = compo2.fork();
			compo.plug("slot", compo1);
			compo.plug("slot", compo2);
			compo.plug("slot", compo3);
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "424242-is--is--is-the answerthe answerthe answer");
			//correct result when using the preplug implementation			
			//assertEquals(rslt.toString() , "42-is-the answer42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}

	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testForkOfPlugged1() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Container with NAry Slot");		
			Component compo = ComponentFactory.newComponent(ts0);
			TemplateDescription ts1 = rep1.getTemplateDescription("test/Plug");		
			Component compo1 = ComponentFactory.newComponent(ts1);
			compo.plug("slot", compo1);
			Component compo2 = compo1.fork();
			compo.plug("slot", compo2);
			Component compo3 = compo2.fork();
			compo.plug("slot", compo3);
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "424242-is--is--is-the answerthe answerthe answer");
			//correct result when using the preplug implementation			
			//assertEquals(rslt.toString() , "42-is-the answer42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testForkOfPlugged2() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Plugging Plug into Container with NAry Slot");		
			Component compo = ComponentFactory.newComponent(ts0);
			Component compo1 = compo.getSubComponent("Container");
			Component compo2 = compo.getSubComponent("Plug");
			Component compo3 = compo2.fork();
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "424242-is--is--is-the answerthe answerthe answer");
			//correct result when using the preplug implementation			
			//assertEquals(rslt.toString() , "42-is-the answer42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testForkOfPlugged3() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Plugging Plug into Container with NAry Slot");		
			Component compo = ComponentFactory.newComponent(ts0);
			Component compo1 = compo.getSubComponent("Container");
			Component compo2 = compo.getSubComponent("Plug");
			Component compo3 = compo2.fork();
			Component compo4 = compo3.fork();
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			assertEquals(rslt.toString() , "42424242-is--is--is--is-the answerthe answerthe answerthe answer");
			//correct result when using the preplug implementation			
			//assertEquals(rslt.toString() , "42-is-the answer42-is-the answer42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testForkOfPlugged4() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Plugging Plug into Container with NAry Slot");		
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			Component compo1 = compo.getSubComponent("Container");
			Component compo2 = compo.getSubComponent("Plug");
			Component compo3 = compo2.fork();
			Component compo4 = compo3.fork();
			assertEquals(rslt.toString() , "4242-is--is-the answerthe answer42-is-the answer42-is-the answer");
			//correct result when using the preplug implementation			
			//assertEquals(rslt.toString() , "42-is-the answer42-is-the answer42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testForkOfPlugged5() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Plugging Plug into Container with NAry Slot");		
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("result");
			Component compo1 = compo.getSubComponent("Container");
			Component compo2 = compo.getSubComponent("Plug");
			Component compo3 = compo2.fork();
			StringBuffer rslt = (StringBuffer) interf.getValue();
			Component compo4 = compo3.fork();
			assertEquals(rslt.toString() , "424242-is--is--is-the answerthe answerthe answer42-is-the answer");
			//correct result when using the preplug implementation			
			//assertEquals(rslt.toString() , "42-is-the answer42-is-the answer42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	 
	/** Tests the triggering of transformers in cycle prone templates.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testCycleAndTriggers() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		TemplateDescription ts0 = rep1.getTemplateDescription("test/Cycle via Trigger");
		Component compo = ComponentFactory.newComponent(ts0);
		assertEquals(compo.getInterface("Un").getValue()	, "Toto est un hros");	
	}
	
	/**
	 * Tests the triggering of transformers on THIS.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testTriggersOnTHIS() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		TemplateDescription ts0 = rep1.getTemplateDescription("test/Triggers on THIS");
		Component compo = ComponentFactory.newComponent(ts0);
		System.out.print("L'affichage correct doit tre 12		:		");
		assertEquals(compo.getInterface("bidon").getValue(), null);
	}
	
	/** Tests the containment behaviour, respect to star ports.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testContainmentAndStart() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		TemplateDescription ts0 = rep1.getTemplateDescription("test/Containment Test");
		Component compo = ComponentFactory.newComponent(ts0);
		assertEquals("ContentContent" , compo.getInterface("text").getValue().toString());	
	}
	
	/** Tests the triggering of transformers in cycle prone templates.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testBindingValue1() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		TemplateDescription ts0 = rep1.getTemplateDescription("test/BindingValue Test");
		Component compo1 = ComponentFactory.newComponent(ts0);
		Component compo2 = compo1.fork();
		ComponentInterface interf1a = compo1.getInterface("toBeBound");
		interf1a.bindValue("Toto est un hros");
		ComponentInterface interf1b = compo1.getInterface("result");
		assertEquals(interf1b.getValue()	, "Toto est un hros");
		ComponentInterface interf2a = compo2.getInterface("toBeBound");
		interf2a.bindValue("Titi est un hros");
		ComponentInterface interf2b = compo2.getInterface("result");
		assertEquals(interf2b.getValue()	, "Titi est un hros");
		Component compo3 = compo2.fork();
		ComponentInterface interf3a = compo3.getInterface("toBeBound");
		interf3a.bindValue("Tutu est un hros");
		ComponentInterface interf3b = compo3.getInterface("result");
		assertEquals(interf3b.getValue()	, "Tutu est un hros");
	}
	
	/**
	 * Tests the triggering of transformers in cycle prone templates.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testBindingProvider1() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		TemplateDescription ts0 = rep1.getTemplateDescription("test/BindingValue Test");
		Mapper mapper = new JavaRuntimeMapper();
		MappingContext mContext = new DefaultMappingContext();

		Component compo1 = ComponentFactory.newComponent(ts0);
		Component compo2 = compo1.fork();
		ComponentInterface interf1a = compo1.getInterface("toBeBound");
		interf1a.bindProvider(new RtExternalValue("toto", "Toto est un hros"), false);
		ComponentInterface interf1b = compo1.getInterface("result");
		assertEquals(interf1b.getValue()	, "Toto est un hros");
		ComponentInterface interf2a = compo2.getInterface("toBeBound");
		interf2a.bindProvider(new RtExternalValue("toto", "Titi est un hros"), false);
		ComponentInterface interf2b = compo2.getInterface("result");
		assertEquals(interf2b.getValue()	, "Titi est un hros");
		Component compo3 = compo2.fork();
		ComponentInterface interf3a = compo3.getInterface("toBeBound");
		interf3a.bindProvider(new RtExternalValue("tutu", "Tutu est un hros"), false);
		ComponentInterface interf3b = compo3.getInterface("result");
		assertEquals(interf3b.getValue()	, "Tutu est un hros");
	}
	
	/**
	 * Method testNullParser
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws KilimException :
	 */
	public void testNullMapper() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		TemplateDescription ts0 = rep.getTemplateDescription("test/Plugging Plug into Container with NAry Slot");		
		//KilimTemplateViewer.viewTemplate(ts0);	
		try {
			//this specific strategy should be defined in order to obtain the expected behaviour ...
			MappingStrategy strgy = new DefaultMappingStrategy(new NullMapper(null));
			KilimConfiguration.setMappingStrategy(strgy);
			Component compo = ComponentFactory.newComponent(ts0);
			Iterator iter = compo.getInterfaces();
			while (iter.hasNext()) {
				ComponentInterface interf = ((ComponentInterface) iter.next());
				System.out.println("interface : " + interf.getQualifiedName());
				interf.getValue();
			}		
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}	
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testOtherMapper1() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Plugging Plug into Container with NAry Slot");		
			TextAreaMapper textMapper = new TextAreaMapper(new JavaRuntimeMapper());
			TreeModelMapper treeMapper = new TreeModelMapper(textMapper);
			KilimTraceTreeModel treeModel = treeMapper.getTraceTree();
			MappingStrategy strgy = new DefaultMappingStrategy(treeMapper);
			KilimConfiguration.setMappingStrategy(strgy);			
			Component compo = ComponentFactory.newComponent(ts0);
			Component compo1 = compo.getSubComponent("Container");
			Component compo2 = compo.getSubComponent("Plug");
			Component compo3 = compo2.fork();
			Component compo4 = compo3.fork();
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			//KilimTraceTreeViewer.viewTrace(treeModel, textMapper.getTextArea());
			assertEquals(rslt.toString() , "42424242-is--is--is--is-the answerthe answerthe answerthe answer");
			//correct result when using the preplug implementation			
			//assertEquals(rslt.toString() , "42-is-the answer42-is-the answer42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}

	/* Tests the triggering of transformers on THIS.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 *
	public void testPrivateAccess() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		TemplateDescription ts0 = rep1.getTemplateDescription("test/Extends Private");
		//KilimTemplateViewer.viewTemplate(ts0);	
		Component compo = ComponentFactory.newComponent(ts0);
		assertEquals(compo.getInterface("public").getValue()	, "private property");	
	}
	*/
}
