/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;

import houseexample.House;
import houseexample.Man;

import javax.swing.JTextArea;
import javax.xml.parsers.ParserConfigurationException;
import junit.framework.TestCase;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentFactory;
import org.objectweb.kilim.model.ComponentInterface;
import org.objectweb.kilim.model.ComponentSlot;
import org.objectweb.kilim.model.RtComponent;
import org.objectweb.kilim.model.RtComponentElement;
import org.objectweb.kilim.model.RtSingleValuePort;
import org.objectweb.kilim.model.mapping.DefaultMappingContext;
import org.objectweb.kilim.model.mapping.NullMapper;
import org.objectweb.kilim.model.mapping.JavaRuntimeMapper;
import org.objectweb.kilim.model.mapping.MappingContext;
import org.objectweb.kilim.model.mapping.TextAreaMapper;
import org.objectweb.kilim.model.mapping.TreeModelMapper;
import org.objectweb.kilim.model.instanciation.DefaultInstanciationStrategy;
import org.objectweb.kilim.model.instanciation.InstanciationStrategy;
import org.objectweb.kilim.model.instanciation.NullInstanciationMger;
import org.objectweb.kilim.tools.KilimComponentViewer;
import org.objectweb.kilim.tools.KilimTemplateViewer;
import org.objectweb.kilim.tools.KilimTraceTreeModel;
import org.objectweb.kilim.tools.KilimTraceTreeViewer;

import org.xml.sax.SAXException;

/**
 * @author delpiano
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class PerformanceTests extends TestCase {

	private ResourceRepository rep;
	private ResourceRepository rep1;

	/**
	 * Constructor for TemplateParserTest.
	 * @param arg0 for junit
	 * @throws SAXException should not happen
	 * @throws ParserConfigurationException should not happen
	 */
	public PerformanceTests(String arg0) throws SAXException, ParserConfigurationException {
		super(arg0);
		TemplateDescriptionParser parser = new TemplateDescriptionParser(true);
		rep = new ResourceRepository(parser);
		rep.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
		rep1 = new ResourceRepository1(parser);
		rep1.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
	}
	
	/**
	 * Method testParsing16.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testPerformance1() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			Runtime rtime = Runtime.getRuntime();
			int nbCompo = 50000;
			System.out.println("total memory " + rtime.totalMemory() + " ======= free memory " + rtime.freeMemory());
			TemplateDescription ts0 = rep.getTemplateDescription("writers/LoopWriter");
			Component[] compo = new Component[nbCompo];
			ComponentInterface[] interf = new ComponentInterface[nbCompo];
			for (int i = 0; i < nbCompo; i++) {
				compo[i] = ComponentFactory.newComponent(ts0);			
				interf[i] = compo[i].getInterface("writer");
				Object res = interf[i].getValue();
				if (rtime.freeMemory() < 100000) {
					System.out.println("iteration n. : " + i + " [nb compo = " + RtComponentElement.getNbComponent() + "]" + " [nb elem = " + RtComponentElement.getNbElement() + "]");
					rtime.gc();
					if (rtime.freeMemory() < 5000) {
						return;
					}
				}
			}
		} catch (Exception ex) { 
			throw new InternalException(ex);
		} catch (OutOfMemoryError ex) {}
	}
	
	public void testPerformance2() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			Runtime rtime = Runtime.getRuntime();
			int nbCompo = 50000;
			System.out.println("total memory " + rtime.totalMemory() + " ======= free memory " + rtime.freeMemory());
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Plugging Plug into Container with NAry Slot");	
			Component[] compo = new Component[nbCompo];	
			ComponentInterface[] interf = new ComponentInterface[nbCompo];
			for (int i =0; i < nbCompo; i++) {
				compo[i] = ComponentFactory.newComponent(ts0);			
				interf[i] = compo[i].getInterface("result");
				StringBuffer rslt = (StringBuffer) interf[i].getValue();
				if (rtime.freeMemory() < 100000) {
					System.out.println("iteration n. : " + i + " [nb compo = " + RtComponentElement.getNbComponent() + "]" + " [nb elem = " + RtComponentElement.getNbElement() + "]");
					rtime.gc();
					if (rtime.freeMemory() < 5000) {
						return;
					}
				}
			}
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		} catch (OutOfMemoryError ex) {}
	}

}
