/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;

import javax.xml.parsers.ParserConfigurationException;
import junit.framework.TestCase;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimException;

import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentFactory;
import org.objectweb.kilim.model.ComponentInterface;

import org.objectweb.kilim.model.mapping.JavaRuntimeMapper;

import org.objectweb.kilim.model.mapping.TextAreaMapper;
import org.objectweb.kilim.model.mapping.TreeModelMapper;

import org.objectweb.kilim.tools.KilimComponentViewer;
import org.objectweb.kilim.tools.KilimTemplateViewer;
import org.objectweb.kilim.tools.KilimTraceTreeModel;
import org.objectweb.kilim.tools.KilimTraceTreeViewer;

import org.xml.sax.SAXException;

/**
 * @author delpiano
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class ViewerTests extends TestCase {
	
	private ResourceRepository rep;
	private ResourceRepository rep1;
	
	/**
	 * Constructor for TemplateParserTest.
	 * @param arg0 for junit
	 * @throws SAXException should not happen
	 * @throws ParserConfigurationException should not happen
	 */
	public ViewerTests(String arg0) throws SAXException, ParserConfigurationException {
		super(arg0);
		TemplateDescriptionParser parser = new TemplateDescriptionParser(true);
		rep = new ResourceRepository(parser);
		rep.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
		rep1 = new ResourceRepository1(parser);
		rep1.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
	}
	
	/**
	 * Tests the template and component viewers.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testViewers() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Container with NAry Slot");
			KilimTemplateViewer.viewTemplate(ts0);	
			Component compo = ComponentFactory.newComponent(ts0);
			KilimComponentViewer.viewComponent(compo);	
			TemplateDescription ts1 = rep1.getTemplateDescription("test/Plug");		
			Component compo1 = ComponentFactory.newComponent(ts1);
			Component compo2 = ComponentFactory.newComponent(ts1);
			compo.plug("slot", compo1);
			compo.plug("slot", compo2);
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			//assertEquals(rslt.toString() , "4242-is--is-the answerthe answer");
			//correct result when using the preplug implementation
			assertEquals(rslt.toString() , "42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Tests the overloading of properties by providers/ports, and ports by properties.
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws ResourceNotFoundException :
	 * @throws KilimException :
	 */
	public void testViewer2() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		try {	
			TemplateDescription ts0 = rep1.getTemplateDescription("test/Plugging Plug into Container with NAry Slot");		
			TextAreaMapper textMapper = new TextAreaMapper(new JavaRuntimeMapper());
			TreeModelMapper treeMapper = new TreeModelMapper(textMapper);
			KilimTraceTreeModel treeModel = treeMapper.getTraceTree();
			Component compo = ComponentFactory.newComponent(ts0);
			KilimTraceTreeViewer.viewTrace(treeModel, textMapper.getTextArea());
			Component compo1 = compo.getSubComponent("Container");
			Component compo2 = compo.getSubComponent("Plug");
			Component compo3 = compo2.fork();
			KilimTraceTreeViewer.updateView();			
			ComponentInterface interf = compo.getInterface("result");
			StringBuffer rslt = (StringBuffer) interf.getValue();
			KilimTraceTreeViewer.updateView();
			assertEquals(rslt.toString() , "424242-is--is--is-the answerthe answerthe answer");
			//correct result when using the preplug implementation			
			//assertEquals(rslt.toString() , "42-is-the answer42-is-the answer42-is-the answer");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
}
