/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;

import javax.swing.JTextArea;
import javax.xml.parsers.ParserConfigurationException;
import junit.framework.TestCase;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentFactory;
import org.objectweb.kilim.model.ComponentInterface;

import org.objectweb.kilim.model.instanciation.DefaultInstanciationStrategy;
import org.objectweb.kilim.model.instanciation.InstanciationStrategy;
import org.objectweb.kilim.model.instanciation.NullInstanciationMger;

import org.objectweb.kilim.model.mapping.DefaultMappingContext;
import org.objectweb.kilim.model.mapping.NullMapper;
import org.objectweb.kilim.model.mapping.JavaRuntimeMapper;
import org.objectweb.kilim.model.mapping.TextAreaMapper;
import org.objectweb.kilim.model.mapping.TreeModelMapper;

import org.objectweb.kilim.tools.KilimTemplateViewer;
import org.objectweb.kilim.tools.KilimTraceTreeModel;
import org.objectweb.kilim.tools.KilimTraceTreeViewer;

import org.xml.sax.SAXException;

/**
 * @author delpiano
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class WritersTest extends TestCase {

	private ResourceRepository rep;
	private ResourceRepository rep1;

	/**
	 * Constructor for TemplateParserTest.
	 * @param arg0 for junit
	 * @throws SAXException should not happen
	 * @throws ParserConfigurationException should not happen
	 */
	public WritersTest(String arg0) throws SAXException, ParserConfigurationException {
		super(arg0);
		TemplateDescriptionParser parser = new TemplateDescriptionParser(true);
		rep = new ResourceRepository(parser);
		rep.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
		rep1 = new ResourceRepository1(parser);
		rep1.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
	}

	/**
	 * Method testParsing3.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testProvidingPortWithInstanciationWithoutArgument() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("writers/WordWriterV1");
			//KilimTemplateViewer.viewTemplate(ts0);	
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "a word");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Method testParsing4.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testProvidingPortWithProviderWithoutArgument() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {	
			TemplateDescription ts0 = rep.getTemplateDescription("writers/WordWriterV2");
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "a word");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Method testParsing5.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testProvidingPortWithInstanciationWithArgument() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {	
			TemplateDescription ts0 = rep.getTemplateDescription("writers/WordWriterV3");
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "new word");
			
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}

	/**
	 * Method testParsing6.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testProvidingPortWithInstanciationWithArgument2() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("writers/WordWriterFrancois");	
			Component compo = ComponentFactory.newComponent(ts0);			
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "Francois");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}

	/**
	 * Method testParsing7.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testPortProvidedByInstanciationAndSetByTrigger() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("writers/WordWriterEst");
			//KilimTemplateViewer.viewTemplate(ts0);
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "est");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}

	/**
	 * Method testParsing18.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testPortProvidedByInstanciationAndSetByTriggerWithDirectFieldAccess() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("writers/WordWriterEstV2");
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "est");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Method testParsing9.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testPortProvidedByInstanciationAndSetByTriggerWithSetterInvocation() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {	
			TemplateDescription ts0 = rep.getTemplateDescription("writers/WordWriterEstV3");
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "est");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}

	/**
	 * Method testParsing10.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testPortProvidedByInstanciationAndSetByTransformer() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {	
			TemplateDescription ts0 = rep.getTemplateDescription("writers/WordWriterPoilu");
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "poilu");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}

	/**
	 * Method testParsing11.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testInstanciation() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("writers/SimpleSentenceWriterV1");
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "Francois est poilu");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}

	/**
	 * Method testParsing12.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testInstanciation2() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("writers/SimpleSentenceWriterV2");	
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res =  interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "Francois est POILU");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}

	/**
	 * Method testParsing13.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testInstanciation3() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("writers/SimpleSentenceWriterV3");
			//InstanciationStrategy strgy = new DefaultInstanciationStrategy(new FullTDInstanciationMger());
			//TextAreaMapper textMapper = new TextAreaMapper(new JavaRuntimeMapper());
			//TreeModelMapper treeMapper = new TreeModelMapper(textMapper);			
			//KilimTraceTreeModel treeModel = treeMapper.getTraceTree();
			//Component compo = ComponentFactory.newComponent(ts0, treeMapper, new DefaultMappingContext());
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res =  interf.getValue();
			//KilimTraceTreeViewer.viewTrace(treeModel, textMapper.getTextArea());
			assertEquals(((writers.Writer) res).getString(), "poilu est poilu");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Method testParsing14.
	 *: @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testArraysAndInstanciation() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("writers/ArrayWriter");
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "poilu est Francois");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}

	/**
	 * Method testParsing15.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testTriggeringOrder() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("writers/ArrayWriterV2");
			InstanciationStrategy strgy = new DefaultInstanciationStrategy();
			//TextAreaMapper textMapper = new TextAreaMapper(new JavaRuntimeMapper());
			//TreeModelMapper treeMapper = new TreeModelMapper(textMapper);			
			//KilimTraceTreeModel treeModel = treeMapper.getTraceTree();
			//Component compo = ComponentFactory.newComponent(ts0, treeMapper, new DefaultMappingContext());
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			String rslt = ((writers.Writer) res).getString();
			assertEquals(rslt, "poilu est Francois");			
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}

	/**
	 * Method testParsing16.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testInstanciationwithMultipleArguments() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("writers/LoopWriter");
			//KilimTemplateViewer.viewTemplate(ts0);	
			Component compo = ComponentFactory.newComponent(ts0);			
			//KilimComponentViewer.viewComponent(compo);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			assertEquals(((writers.Writer) res).getString(), "Francois est poilu Francois est poilu Francois est poilu");
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Method testParsing10.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testOfNullProvider() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {	
			TemplateDescription ts0 = rep.getTemplateDescription("writers/SimpleSentenceWriterV3");
			//InstanciationStrategy strgy = new DefaultInstanciationStrategy(new FullTDInstanciationMger());
			//TreeModelMapper treeMapper = new TreeModelMapper(new JavaNullMapper(null));			
			//KilimTraceTreeModel treeModel = treeMapper.getTraceTree();
			//Component compo = ComponentFactory.newComponent(ts0, treeMapper, new DefaultMappingContext());
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("writer");
			Object res = interf.getValue();
			//KilimTraceTreeViewer.viewTrace(treeModel, new JTextArea());
		} catch (Exception ex) { 
			throw new InternalException(ex);
		}
	}
}
