package org.objectweb.kilim;

import java.util.Properties;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.wrapper.log4j.MonologLoggerFactory;
/**
 * @author delpiano
 */
public class KilimLoggerFactory implements LoggerFactory {

	private static LoggerFactory singleton;

	static {
		
		Properties prprts = System.getProperties();
		String log = prprts.getProperty("org.objectweb.kilim.log");
		boolean log_on = false;
		if (log != null) {
			log_on = log.equals("true");
		}
		
		if (log_on) {
			try {
				singleton = new MonologLoggerFactory();
			} catch (Throwable e) {
				System.out.println("KILIM WARNING : No Logger found, disabling logging functionality for Kilim module.");
				singleton = new NullLoggerFactory();
			}
		} else {
			singleton = new NullLoggerFactory();	
		}
			
	}

	/**
	 * returns the LoggerFactory.
	 * @return LoggerFactory
	 */
	public static LoggerFactory getSingleton() {
		return singleton;
	}

	/**
	 * @see org.objectweb.util.monolog.api.LoggerFactory#getLogger(String)
	 */
	public Logger[] getLoggers() {
		return singleton.getLoggers();
	}

	/**
	 * @see org.objectweb.util.monolog.api.LoggerFactory#getLogger(String)
	 */
	public Logger getLogger(String key) {
		return singleton.getLogger(key);
	}

	/**
	 * @see org.objectweb.util.monolog.api.LoggerFactory#getLogger(String, String)
	 */
	public Logger getLogger(String key, String resource_bundle_name) {
		return singleton.getLogger(key, resource_bundle_name);
	}

	/**
	 * @see org.objectweb.util.monolog.api.LoggerFactory#getResourceBundleName()
	 */
	public String getResourceBundleName() {
		return singleton.getResourceBundleName();
	}

	/**
	 * @see org.objectweb.util.monolog.api.LoggerFactory#setResourceBundleName(String)
	 */
	public void setResourceBundleName(String resource_bundle_name) {
		singleton.setResourceBundleName(resource_bundle_name);
	}

}
