/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 * Superinterface of all of {@link Template}-contained constructs.
 * {@see Template}
 */
public abstract class InlinedElement extends TemplateElementImpl implements BasicElement {
	//private TemplateDescription containingTemplate;
	private boolean isProvider;
	private boolean isTransformer;
	private String name;
		
	/**
	 * The protected constructor of an inlined element.
	 * @param isP : is true when element is a provider
	 * @param isT : is true when the element is a transformer
	 * @param aTemplate : the templatye in which the element is defined.
	 * @throws KilimException :
	 */
	protected InlinedElement(boolean isP, boolean isT, TemplateDescription aTemplate) throws KilimException {
		isProvider = isP;
		isTransformer = isT;
		setContainingTemplate(aTemplate);
		name = "%inlined" + KILIM.getUniqueIndex();
	}
	
	/**
	 * Every Kilim element has a name  (for debugging purposes). This method sets the local name of an inlined element.
	 * @return String
	 */
	public String getLocalName()  {
		return name;
	}
	
	/**
	 * sets the local name of an inlined element. This name is for debugging purpouse only.
	 * @param aName : the local name
	 * @throws KilimException : never generated.
	 */
	public void setLocalName(String aName) throws KilimException {
		name = aName;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#setStatus(int)
	 */
	public void setStatus(int aStatus) throws KilimException {
		throw new KilimException("attempt to set a status to an inlined element in template " + getContainingTemplate().getName());
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#getStatus()
	 */
	public int getStatus() throws KilimException {
		throw new KilimException("attempt to get the status of an inlined element in template " + getContainingTemplate().getName());
	}
			
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#providesValue()
	 */
	public boolean providesValue() {
		return isProvider;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#performsAction()
	 */
	public boolean performsAction() {
		return isTransformer;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#isEventSource()
	 */
	public boolean isEventSource() {
		return false;
	}
}