/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.description;

import java.util.Iterator;
import java.util.LinkedList;

/**
 * @author horn
 * Abstract class introduced for grouping common kiilim constants.
 * @see Port
 */
public abstract class KILIM  {

	/** integer value identifying a port */	
   public static final int PORT = 1;
   	/** integer value identifying a property */	
   public static final int PROPERTY = 2;

	/** integer value identifying a getter */	   
   public static final int GETTER = 3;
	/** integer value identifying a setter */	
   public static final int SETTER = 4;
	/** integer value identifying a method */	
   public static final int METHOD = 5;
	/** integer value identifying a constructor */	
   public static final int CONSTRUCTOR = 6;

	/** integer value identifying a reference */	  
   public static final int REFERENCE = 7;
	/** integer value identifying a class provider */	
   public static final int CLASS = 8;
 	/** integer value identifying a array provider */	
   public static final int ARRAY = 9;
 
 	/** integer value identifying an event source */	
   public static final int EVENT_SOURCE = 10;

	/** integer value identifying a null element */	
   public static final int NULL_ELEMENT = 11;
   	/** integer value identifying a provider */	
   public static final int PROVIDER = 12;
   	/** integer value identifying a transformer */	
   public static final int TRANSFORMER = 13;

	/** return value for empty list */
	public static final LinkedList EMPTY_LIST = new LinkedList();
	/** return value for empty iterator */
	public static final Iterator EMPTY_ITERATOR = EMPTY_LIST.listIterator();
	
	/** private status */
	public static final int PRIVATE = 1;
	/** protected status */
	public static final int PROTECTED = 2;
	/** public status */
	public static final int PUBLIC = 3;
	/** undefined status */
	public static final int UNDEFINED_STATUS = 0;
	
	/** a primitive type */
	public static final int BOOLEAN = 1;
	/** a primitive type */
	public static final int CHAR = 2;
	/** a primitive type */
	public static final int BYTE = 3;
	/** a primitive type */
	public static final int SHORT = 4;
	/** a primitive type */
	public static final int INT = 5;
	/** a primitive type */
	public static final int LONG = 6;
	/** a primitive type */
	public static final int FLOAT = 7;
	/** a primitive type */
	public static final int DOUBLE = 8;
	/** a primitive type */
	public static final int STRING = 9;
	/** a primitive type */
	public static final int OBJECT = 10;	
	/** undefined status */
	public static final int UNDEFINED_TYPE = 0;

	/** name of class wrappers for primitive types */	
	public static final String[] CLASS_NAME = { "", "java.lang.Boolean", "java.lang.Character", "java.lang.Byte", "java.lang.Short",
																			"java.lang.Integer", "java.lang.Long", "java.lang.Float", "java.lang.Double", "java.lang.String", "java.lang.Object" };
		
	private static int uniqueIndex = 0;
		
	/**
	 * generates a unique index. This method is used to generate unique names in Kilim (for unnamed inlined elements for examples): 
	 * unique indexes are added as suffixes to predefined names.
	 * @return int
	 */
	public static int getUniqueIndex() {
		return uniqueIndex++;
	}
	
   /**
    * returns the Class class corresponding to a  type.  If it is a primitive type name, convert it to corresponding wrapper class.
    * @param name : the name of the class.
    * @return Class
    * @throws ClassNotFoundException :
    */
   public static Class getClassFromName(String name) throws ClassNotFoundException {
      if (name.equals("boolean")) {
         return boolean.class;
      } else if (name.equals("byte")) {
         return byte.class;
      } else if (name.equals("short")) {
         return short.class;
      } else if (name.equals("int")) {
         return int.class;
      } else if (name.equals("long")) {
         return long.class;
      } else if (name.equals("float")) {
         return float.class;
      } else if (name.equals("double")) {
         return double.class;
      } else if (name.equals("char")) {
         return char.class;
      } else {
         return Class.forName(name);
      }     
   }
}