/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * Named elements are template elements that have a name and a status 
 * (which is one of KILIM.PUBLIC, KILIM.PROTECTED, KILIM.PRIVATE).
 * @author horn
 */
public interface NamedElement extends TemplateElement {
	/**
	 * returns the name of the element.
	 * @return String
	 */
	String getLocalName();

	/**
	 * sets a name to the element.
	 * @param aName : the name to be set
	 * @throws KilimException : generated when aName is null.
	 */
	void setLocalName(String aName) throws KilimException;
	
	/**
	 * returns the status of the element.
	 * @return int. The value is one of KILIM.PRIVATE, KILIM.PROTECTED, KILIM.PUBLIC.
	 */
	int getStatus();

	/**
	 * sets the element status.
	 * @param aStatus : The value must be one of KILIM.PRIVATE, KILIM.PROTECTED, KILIM.PUBLIC.
	 * @throws KilimException  generated if the value for aStatus is not valid.
	 */
	void setStatus(int aStatus) throws KilimException;
}
