package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
public class Provider extends BasicNamedElementImpl {
	private BasicElement source;

	/**
	 * a public constructor for providers.
	 * @param aName : the local name of the provider
	 * @param aStatus : the status of the provider. It should be one of KILIM.PRIVATE, KILIM.PROTECTED, KILIM.PUBLIC.
	 * @param isT : is true if the provider can also be used as a transformer.
	 * @param aTemplate : the template in which the provider is defined
	 * @throws KilimException : generated if aName or aTemplate is null or if the value for aStatus is invalid.
	 */
	public Provider(String aName, int aStatus, boolean isT, TemplateDescription aTemplate) throws KilimException {
		super(aName, aStatus, true, isT, aTemplate);
		source = new NullElement(aName, aStatus, true, isT, aTemplate);
	}
		
	/**
	 * a public constructor for providers. The constructor equivalent to this(aName, aStatus, false, aTemplate);
	 * @param aName : the local name of the provider
	 * @param aStatus : the status of the provider. It should be one of KILIM.PRIVATE, KILIM.PROTECTED, KILIM.PUBLIC.
	 * @param aTemplate : the template in which the provider is defined
	 * @throws KilimException : generated if aName or aTemplate is null or if the value for aStatus is invalid.
	 */
	public Provider(String aName, int aStatus, TemplateDescription aTemplate) throws KilimException {
		super(aName, aStatus, true, false, aTemplate);
	}
	
	/**
	 * @see org.objectweb.kilim.description.BasicElement#getKind()
	 */
	public int getKind() {
		return KILIM.PROVIDER;
	}

	/**
	 * sets the source of the provider.
	 * @param aSource : the element providing the value.
	 * @throws KilimException : generated when the element provided as an argument does not provide a value.
	 */
	public void setSource(BasicElement aSource) throws KilimException {
		if (aSource != null && !aSource.providesValue()) {
			throw new KilimException("illegal attempt to set a non provider inline to the provider " + getLocalName());
		}
		source = aSource;
	}
	
	/**
	 * returns the source.
	 * @return InlinedElement
	 * @throws KilimException :
	 */
	public BasicElement getSource() throws KilimException {
		return source;
	}
	
		/**
	 * @see org.objectweb.kilim.description.TemplateElement#setLocalName(String)
	 */
	public void setLocalName(String aName) throws KilimException {
		String oName = getLocalName();
		super.setLocalName(aName);
		TemplateDescription template = getContainingTemplate();
		if (template != null) {
			template.removeLocalProvider(oName);
			template.addLocalProvider(this);
		}
	}
		
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#setStatus(int)
	 */
	public void setStatus(int aStatus) throws KilimException {
		super.setStatus(aStatus);
		TemplateDescription template = getContainingTemplate();
		if (template != null) {
			template.removeLocalProvider(getLocalName());
			template.addLocalProvider(this);
		}
	}
}