package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public abstract class TpAccessor extends InlinedElement {		
	protected BasicElement support;
	protected String fieldName;
	protected boolean isStatic;

	/**
	 * The public constructor for TpAccessor.
	 * @param aSupport : the object the accessor should be applied to.
	 * @param aField : the name of the field to be accessed
	 * @param jStatic : true if the accessor is static.
	 * @param isP : true if the accessor is a getter.
	 * @param isT : true if the accessor is a setter.
	 * @param aTemplate : the template in which the accessor is defined.
	 * @throws KilimException : generated if aTemplate is null.
	 */
	protected TpAccessor(BasicElement aSupport, String aField, boolean jStatic, boolean isP, boolean isT, TemplateDescription aTemplate) throws KilimException {
		super(isP, isT, aTemplate);
		support = aSupport;
		fieldName = aField;
		isStatic = jStatic;
	}
	
	/**
	 * Method isStatic.
	 * @return boolean
	 */
	public boolean isStatic() {
		return isStatic;
	}
	
	/**
	 * @see org.objectweb.kilim.description.libs.FieldAction#isStatic(boolean)
	 */
	public void isStatic(boolean jStatic) {
		isStatic = jStatic;
	}
	
	/**
	 * @see org.objectweb.kilim.description.libs.FieldAction#getSupport()
	 */
	public BasicElement getSupport() {
		return support;
	}
	
	/**
	 * sets the support i.e. te object the accessor will be applied to.
	 * @param aElement : the element that is the support of the accessor.
	 * @throws KilimException : generated if aElement is null or does not provide a value. 
	 */
	public void setSupport(BasicElement aElement) throws KilimException {
		if (aElement == null) {
			throw new KilimException("attempt to set a null support to a field transformer in template " + getContainingTemplate().getName());
		}
		
		if (!aElement.providesValue()) {
			throw new KilimException("attempt use a non provider to set the support of an accessor in template " + getContainingTemplate().getName());
		}
		support = aElement;
	}
	
	/**
	 * returns the name of the filed to be accessed.
	 * @return String
	 */
	public String getFieldName() {
		return fieldName;
	}
	
	/**
	 * sets the name of the field to be accessed.
	 * @param aFieldName : the name of the field.
	 * @throws KilimException : generated if aFieldName is null.
	 */
	public void setFieldName(String aFieldName) throws KilimException {
		if (fieldName == null) {
			throw new KilimException("attempt to set a null name to a field of a transformer in template " + getContainingTemplate().getName());
		}
		fieldName = aFieldName;
	}	
}