package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */

public class TpConstructor extends TpMethod  {

	/**
	 * A public constructor for "constructor" descriptions.
	 * @param aSupport : the support of the constructor. It is a ClassSource
	 * @param isP : is true if the constructor can be used as a provider (is generally true).
	 * @param isT : is true if the constructor can be used as a transformer (through its side effects).
	 * @param aTemplate : the template in which the constructor is declared.
	 * @throws KilimException : generated if aSupport or aTemplate is null.
	 */
	public TpConstructor(BasicElement aSupport, boolean isP, boolean isT, TemplateDescription aTemplate) throws KilimException {
		super(aSupport, "<init>", true, isP, isT, aTemplate);	
	}
	
	/**
	 *A public constructor  for "constructor" descriptions. It is strictly equivalent to TpConstructor(aSupport, true, false, aTemplate);
	 * @param aSupport : the support of the constructor. It is a ClassSource
	 * @param aTemplate : the template in which the constructor is declared.
	 * @throws KilimException : generated if aSupport or aTemplate is null.
	 */
	public TpConstructor(BasicElement aSupport, TemplateDescription aTemplate) throws KilimException {
		this(aSupport, true, false, aTemplate);	
	}
		
	/**
	 * @see org.objectweb.kilim.description.MethodSourceImpl#isStatic(boolean)
	 */
	public void isStatic(boolean jStatic) throws KilimException {
		throw new KilimException("attempt to set a static qualifier on a ctor in template " + getContainingTemplate().getName());
	}
	
	/**
	 * @see org.objectweb.kilim.description.BasicElement#getKind()
	 */
	public int getKind() {
		return KILIM.CONSTRUCTOR;
	}
}
