package org.objectweb.kilim.helpers;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.KilimConfiguration;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.mapping.JavaLoggerMapper;
import org.objectweb.kilim.model.mapping.JavaRuntimeMapper;
import org.objectweb.kilim.model.mapping.Mapper;

import org.objectweb.kilim.model.instanciation.DefaultInstanciationStrategy;
import org.objectweb.kilim.model.instanciation.InstanciationStrategy;

import org.objectweb.kilim.repository.ClassLoaderResourceLoader;
import org.objectweb.kilim.repository.ResourceNotFoundException;
import org.objectweb.kilim.repository.ResourceRepository;
import org.objectweb.kilim.repository.TemplateDescriptionParser;

/**
 * A helper class providing simplified ways of creating new Template instances.
 * @author delpiano
 */

public class KilimHelper {
	
	private static TemplateDescriptionParser parser = new TemplateDescriptionParser(true);
	private static ResourceRepository rep = new ResourceRepository(parser);
	
	/**
	 * Creates a new instance of the Template named <code>name</code>, the definition of which will
	 * be loaded with the ClassLoader which loaded the class <code>clazz</code>.
	 * @param name the name of the Template that will be instantiated.
	 * @param clazz the class used to determine the ClassLoader used to load the Template definition.
	 * @return the component instance of the Template <code>name</code>
	 * @throws ResourceNotFoundException :
	 * @throws KilimException : 
	 */
	public static Component newComponent(String name , Class clazz) throws ResourceNotFoundException, KilimException {		
		rep.setResourceLoader(new ClassLoaderResourceLoader(clazz.getClassLoader()));
		TemplateDescription ts0 = rep.getTemplateDescription(name);
		Component result = org.objectweb.kilim.model.ComponentFactory.newComponent(ts0);
		return result;
		
	}

	/**
	 * Returns the Template named <code>name</code>, the definition of which will
	 * be loaded with the ClassLoader which loaded the class <code>clazz</code>.
	 * @param name the name of the Template that will be instantiated.
	 * @param clazz the class used to determine the ClassLoader used to load the Template definition.
	 * @return the Template <code>name</code>
	 * @throws ResourceNotFoundException :
	 * @throws KilimException : 
	 */
	public static TemplateDescription getTemplate(String name , Class clazz) throws ResourceNotFoundException, KilimException {
		parser = KilimConfiguration.getTemplateDescriptionParser();
		rep.setResourceLoader(new ClassLoaderResourceLoader(clazz.getClassLoader()));
		TemplateDescription ts0 = rep.getTemplateDescription(name);	
		return ts0;	
	}

	/**
	 * Creates a new instance of the Template named <code>name</code>, the definition of which will
	 * be loaded with the ClassLoader which loaded the class <code>clazz</code>.
	 * @param name the name of the Template that will be instantiated.
	 * @param aStrategy is the instanciation strategy to be used.
	 * @param clazz the class used to determine the ClassLoader used to load the Template definition.
	 * @return the component instance of the Template <code>name</code>
	 * @throws ResourceNotFoundException :
	 * @throws KilimException : 
	 */
	public static Component newComponent(String name , Class clazz, InstanciationStrategy aStrategy) throws ResourceNotFoundException, KilimException {			
		rep.setResourceLoader(new ClassLoaderResourceLoader(clazz.getClassLoader()));
		TemplateDescription ts0 = rep.getTemplateDescription(name);
		KilimConfiguration.setInstanciationStrategy(aStrategy);
		Component result = org.objectweb.kilim.model.ComponentFactory.newComponent(ts0);
		return result;
	}

}
