/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
	
public interface ComponentSource extends ComponentElement {
	/**
	 * This method has a behaviour that depends on the exact nature of the value source.
	 * In case of properties, it simply returns the property value associated to it.
	 * In case of providers, it (re)executes the corresponding method, constructor, getter, ..... and returns the (new) obtained result.
	 * In case of unary ports two cases should be distinguished : 1) getValue() has not been previously invoked and no value is thus associated to the source.
	 * getValue() then results in the execution of the corresponding method, constructor, getter, ...... The obtained result is stored and returned after execution
	 * of the "bind" triggers.. 2) getValue() has been previously invoked. The stored result is returned.
	 * @return Object : the reference of the object provided by the source.
	 * @throws KilimException : the exception is generated when the method is invoked on unbound references or nary ports.
	 */
	Object getValue() throws KilimException;
	
	/**
	 * returns whether a value has been evaluated and stored in the source. The behaviour of this method is
	 * highly dependant on the real nature of the source. It always returns false when invoked on a provider.
	 * @return boolean
	 * @throws KilimException : the exception is generated when the method is invoked on unbound references or nary ports.
	 */
	boolean hasValue()  throws KilimException;
}