/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Stack;

import org.objectweb.kilim.KilimException;

/**
 * RtExternalValues are specific sources referencing external system entities i.e. entities not described by Kilim descriptors.
 *  RtExternalValues can be seen as special properties referencing objects without descriptors and runtime objects.
 * Remark :  Next Kilim implementation should define this class as an extension of ComponentProperty. 
 * @author horn
 */	
public class RtExternalValue extends RtComponentSource {
	private String name;
	private Object value;
		
	/**
	 * The public constructor for external values (i.e. objects without kilim descriptor). 
	 * @param aName : the name of the external value
	 * @param aValue : the object reference.
	 */
	public RtExternalValue(String aName, Object aValue) {
		super(null, null);
		name = aName;
		value = aValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getLocalName()
	 */
	public String getLocalName() {
		return name;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getValue()
	 */
	public Object getValue() throws KilimException {
		Object resultValue = null;
		if (mappingContext != null) {
			mappingContext.getCallStack().push(this);
		}
		mapper.enterContext(mappingContext);
		resultValue = mapper.getExternalValue(value, mappingContext);

		if (mappingContext != null) {
			mappingContext.getCallStack().pop();
		}
		mapper.leaveContext(mappingContext);
		return resultValue;
	}	
							
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#isEventSource()
	 */
	public boolean isEventSource() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getEventSource()
	 */
	public Object getEventSourceValue() throws KilimException {
		throw new KilimException("attempt to get the event source of element ");
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#setEventSource(Object)
	 */
	public void setEventSourceValue(Object aSource) throws KilimException {
		throw new KilimException("attempt to set the event source to a element ");
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#checkValue()
	 */
	public boolean hasValue() throws KilimException {
		return true;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#checkValue(Stack)
	 */
	public boolean checkValue(Stack exclude) throws KilimException {
		return true;
	}
}