/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Stack;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
	
public interface RuntimeSource extends RuntimeElement {	
	/**
	 * Method getValue.
	 * @return Object : the reference of the object provided by the source.
	 * @throws KilimException : the exception is generated when the method is invoked on unbound references 
	 * or on illegal elements.
	 */
	Object getValue() throws KilimException;
	
	/**
	 * returns whether the source has a value. The behaviour depends on the nature of the source.
	 * It always returns false when invoked on tagged providers, for example.
	 * @return boolean
	 * @throws KilimException :
	 */
	boolean hasValue() throws KilimException;
	
	/**
	 * returns whether a value can be obtained without having to create new objects 
	 * @param exclude : the exclude stack contains objects beiing in the process of being built.
	 * @return boolean
	 * @throws KilimException :
	 */
	boolean checkValue(Stack exclude) throws KilimException;	
	
	/**
	 * returns the target element. This method has been introduced to deal with provider references.
	 * @return RuntimeElement
	 * @throws KilimException : generated when applied to an uninitialized provider reference.
	 */
	RuntimeElement getTarget() throws KilimException;
	
	/**
	 * adds a new listener to the value source.
	 * @param aInterface : the collection port to be added to the list of listeners.
	 * @throws KilimException : the exception is generated when the method is invoked on unbound references 
	 * or on illegal elements.
	 */
	void addInterfaceListener(RtCollectionPort aInterface) throws KilimException;
	
	/**
	 * removes a listener from the value source.
	 * @param aInterface : the collection port to be removed from the list of listeners.
	 * @throws KilimException : the exception is generated when the method is invoked on unbound references 
	 * or on illegal elements.
	 */
	void removeInterfaceListener(RtCollectionPort aInterface) throws KilimException;
		
	/**
	 * returns whether the source is an event source..
	 * @return boolean : is true when the source is the current event source.
	 */
	boolean isEventSource();
	
	/**
	 * sets the EventSourceValue.
	 * @param aValue :
	 * @throws KilimException : the exception is generated when the method is invoked on unbound references 
	 * or on illegal elements.
	 */
	void setEventSourceValue(Object aValue) throws KilimException;
	
	/**
	 * Method gets the EventSourceValue.
	 * @return Object
	 * @throws KilimException : the exception is generated when the method is invoked on unbound references 
	 * or on illegal elements.
	 */
	Object getEventSourceValue() throws KilimException;
}
