/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Iterator;
import java.util.LinkedList;

import org.objectweb.kilim.description.KILIM;

/**
 * @author horn
 */
public class RuntimeTrigger {
	private int eventKind;
	private LinkedList actions;
	private RtComponentInterface eventSource;
	
	/**
	 * the public constructor for RuntimeTrigger.
	 * @param aEventKind : the kind of event which should be one of Trigger.BIND or Trigger.UNBIND.
	 * @param aSource : the event source.
	 */
	public RuntimeTrigger(int aEventKind, RtComponentInterface aSource) {
		eventKind = aEventKind;
		eventSource = aSource;
	}
	
	/**
	 * returns the event kind associated to the trigger. It is one of Trigger.BIND or Trigger.UNBIND.
	 * @return int
	 */
	public int getEventKind() {
		return eventKind;
	}
	
	/**
	 * returns the event source.
	 * @return ComponentInterface
	 */
	public RtComponentInterface getEventSource() {
		return eventSource;
	}
		
	/**
	 * @see org.objectweb.kilim.description.Trigger#addTransformer(Transformer)
	*/
	public void addTransformer(RuntimeAction aAction) {
		if (actions == null) {
			actions = new LinkedList();
		}
		actions.add(aAction);
	}
	
	/**
	 * returns as an iterator the transformers associated to the the trigger.
	 * @return Iterator
	 */
	public Iterator getTransformers() {
		if (actions == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return actions.listIterator();
	}
}
