/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.mapping;

import java.io.Serializable;
import java.util.Stack;

import org.objectweb.kilim.model.ComponentElement;

/**
 * A mapping context is a container for all informations useful when performing the effective task associated to providers and transformers.
 * This simple implementation provides two stacks : (1) callStack, a stack automatically managed by the Kilim infrastructure,  which contains 
 * the RtComponentSources mirroring on the execution stack, (2) contextstack, a stack specifically managed by mappers.
 * calls to getValue() and execute() methods.
 * @author horn
 */	
public class DefaultMappingContext implements MappingContext, Serializable {
	private Stack callStack = new Stack();
	private Stack contextStack = new Stack();
	private ComponentElement currentElement;
		
	/**
	 * Method DefaultMappingContext.
	 */
	public DefaultMappingContext() { }
	
	/**
	 * Method getContextStack. It returns the contextStack.
	 * @return Stack
	 */
	public Stack getContextStack() {
		return contextStack;
	}
	
	/**
	 * Method getCallStack. This method returns the callStack.
	 * @return Stack
	 */
	public Stack getCallStack() {
		return callStack;
	}
}
