/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.repository;

import java.io.InputStream;

import org.objectweb.kilim.KilimLoggerFactory;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Implementation of {@link ResourceLoader} using a ClassLoader
 * @author dutoo
 */
public class ClassLoaderResourceLoader implements ResourceLoader {
   
   private static boolean DEBUG_ON = false;
   private static Logger logger = KilimLoggerFactory.getSingleton().getLogger("org.objectweb.kilim.repository");
   
   /**
    * Creates a new ClassLoaderResourceLoader using the provided class loader.
    * @param resourceLoader the class loader to be used
    */
   public ClassLoaderResourceLoader(ClassLoader resourceLoader) {
      this.resourceLoader = resourceLoader;
   }
   
   /**
    * @see org.objectweb.kilim.repository.ResourceLoader#getResource(String)
    */
   public InputStream getResource(String resourceName) throws ResourceNotFoundException {
      if (DEBUG_ON && logger.isLoggable(BasicLevel.DEBUG)) {
      	logger.log(BasicLevel.DEBUG, "Getting resource \"" + resourceName + "\".");
      }
      InputStream res = resourceLoader.getResourceAsStream(resourceName);
      
      if (res == null) {
         throw new ResourceNotFoundException(resourceName);
      }
      
      return res;
   }

   private ClassLoader resourceLoader;

}
