/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.repository;

import java.io.IOException;
import javax.xml.parsers.ParserConfigurationException;

import org.objectweb.kilim.KilimException;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * @author dutoo, horn
 * Parsing errors handling proxy. This includes some description model specific exceptions
 * and some SAX specific ones.
 */
public interface ParserErrorHandler {
	
	/**
	 * Method setTemplateName.
	 * @param aName : the name of the template being parsed.
	 */
	void setTemplateName(String aName);
   
   /**
    * @param ioex the exception to be handled
    */
   void handleIOException(IOException ioex);
   /**
    * @param saxex the exception to be handled
    */
   void handleSAXException(SAXException saxex);
   /**
    * @param pcex the exception to be handled
    */
   void handleParserConfigurationException(ParserConfigurationException pcex);
   /**
    * @param saxpex the exception to be handled
    */
   void handleWarningSAXParseException(SAXParseException saxpex);
   /**
    * @param saxpex the exception to be handled
    */
   void handleErrorSAXParseException(SAXParseException saxpex);
   /**
    * @param saxpex the exception to be handled
    */
   void handleFatalSAXParseException(SAXParseException saxpex);

	/**
	 * Method handleKilimException.
	 * @param ex :
	 */
   void handleKilimException(KilimException ex);
}
