package org.objectweb.kilim.tools;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentInterface;
import org.objectweb.kilim.model.ComponentSlot;
import org.objectweb.kilim.model.RtComponentElement;
import org.objectweb.kilim.model.RtCollectionPort;


/**
 * @author delpiano
 */
public class KilimComponentViewer {
	
	private static final String WINDOW_NAME = "Kilim Component Viewer";
	
	private static ActionListener action_listener = new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
			synchronized (this) {
				this.notifyAll();
			}
		}
	};
	
	private static JFrame view_frame = new JFrame();
	
	static {
		view_frame.setTitle(WINDOW_NAME);
		view_frame.addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent evt) {
				synchronized (action_listener) {
					action_listener.notifyAll();
				}
			}
		});
	}
	
	private static JTree tree = new JTree();
	private static TreeModel treeModel;
	
	/**
	 * Method viewComponent.
	 * @param cmpnnt : the component to be displayed.
	 */
	public static void viewComponent(Component cmpnnt) {
		tree.setCellRenderer(new KilimComponentTreeCellRenderer());
		
		Container cntnr = view_frame.getContentPane();
		cntnr.setLayout(new BorderLayout());
		JScrollPane scrllpn = new JScrollPane(tree);
		cntnr.add(BorderLayout.CENTER, scrllpn);
		view_frame.setSize(500, 600);
		
		ToolTipManager.sharedInstance().registerComponent(tree);
		
		JButton button = new JButton("continue");
		button.addActionListener(action_listener);
		cntnr.add(BorderLayout.SOUTH, button);
		
		TreeModel treeModel = new KilimComponentTreeModel(cmpnnt);
		tree.setModel(treeModel);
		tree.repaint();
		
		String tmplt_nm = "error";
		tmplt_nm = cmpnnt.getTemplate().getName();
		
		view_frame.setTitle(WINDOW_NAME + " - " + tmplt_nm);
		
		view_frame.setVisible(true);

		synchronized (action_listener) {
			try {
				action_listener.wait();
			} catch (InterruptedException exp) {
				exp.printStackTrace();
			}
		}
	}
	
	/**
	 * Method updateView.
	 */
	public static void updateView() {
		//tree.setModel(null);
		//tree.setModel(treeModel);		
		view_frame.validate();
		view_frame.repaint();

		synchronized (action_listener) {
			try {
				action_listener.wait();
			} catch (InterruptedException exp) {
				exp.printStackTrace();
			}
		}
	}
}

class KilimComponentTreeCellRenderer extends DefaultTreeCellRenderer {

	private static final String[] ICON_NAMES = {"provider.gif", "transformer.gif", "template_instance.gif",
												"binding.gif", "port.gif", "template.gif", "property.gif",
												"multiPort.gif", "slot.gif"};
												
	private static final Icon[] ICONS = new Icon[ICON_NAMES.length];
	
	static {
		for (int i = 0; i < ICON_NAMES.length; i++) {
			ICONS[i] = getIcon(ICON_NAMES[i]);
		}
	}

	/**
	 * @see javax.swing.tree.TreeCellRenderer#getTreeCellRendererComponent(JTree, Object, boolean, boolean, boolean, int, boolean)
	 */
	public java.awt.Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded,
															boolean leaf, int row, boolean hasFocus) {
																
		super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
		
		if (value instanceof Component) {
			if (ICONS[2] != null) {
				setIcon(ICONS[2]);
			}
			setText(((RtComponentElement) value).getLocalName());
			
		} else if (value instanceof RtComponentElement) {
			
			String addtnl_text = "";
			
			if (value instanceof ComponentInterface) {
				
				ComponentInterface cmpnnt_intrf = (ComponentInterface) value;
				
				if (cmpnnt_intrf.isProperty()) {
					if (ICONS[6] != null) {
						setIcon(ICONS[6]);
					}
				} else if (cmpnnt_intrf.isProvider()) {
					if (ICONS[0] != null) {
						setIcon(ICONS[0]);
					}				
				} else if (cmpnnt_intrf.isSingleValuePort()) {
					if (ICONS[4] != null) {
						setIcon(ICONS[4]);
					}
				} else if (cmpnnt_intrf.isCollectionPort()) {
					if (ICONS[7] != null) {
						setIcon(ICONS[7]);
					}
				}
				
				try {
					if (!(value instanceof RtCollectionPort)) {
						addtnl_text = ((ComponentInterface) value).getValue() + "";
					}
				} catch (KilimException exp) {
					exp.printStackTrace();
				}
			}
			
			if (value instanceof ComponentSlot) {	
				if (ICONS[8] != null) {
					setIcon(ICONS[8]);
				}
			}		

			setToolTipText(addtnl_text);

			if (addtnl_text.length() > 40) {
				addtnl_text = addtnl_text.substring(0 , 40) + "...";
			}
			setText(((RtComponentElement) value).getLocalName() + " [" + addtnl_text + "]");

		}
		

		
		return this;
	}
	
	private static ImageIcon getIcon(String name) {
		URL url = ClassLoader.getSystemClassLoader().getSystemResource(name);
		if (url != null) {
			Image img = Toolkit.getDefaultToolkit().getImage(url);
			if (img != null) {
				return new ImageIcon(img);
			} else {
				return null;
			}
		} else {
			return null;
		}
	}
}
