package org.objectweb.kilim.tools;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.Iterator;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;

import org.objectweb.kilim.description.Arity;
import org.objectweb.kilim.description.Binding;
import org.objectweb.kilim.description.Instance;
import org.objectweb.kilim.description.NamedElement;
import org.objectweb.kilim.description.Plug;
import org.objectweb.kilim.description.Port;
import org.objectweb.kilim.description.Property;
import org.objectweb.kilim.description.Provider;
import org.objectweb.kilim.description.Slot;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.description.TemplateElementImpl;
import org.objectweb.kilim.description.TpMethod;
import org.objectweb.kilim.description.Transformer;
import org.objectweb.kilim.description.Trigger;

/**
 * @author delpiano
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class KilimTemplateViewer {
	
	private static final String WINDOW_NAME = "Kilim Template Viewer";
	
	private static ActionListener action_listener = new ActionListener(){
		public void actionPerformed(ActionEvent evt) {
			synchronized (this) {
				this.notifyAll();
			}
		}
	};
	
	private static JFrame view_frame = new JFrame();
	
	static {
		view_frame.setTitle(WINDOW_NAME);
		view_frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				synchronized (action_listener) {
					action_listener.notifyAll();
				}
			}
		});
	}
	
	private static JTree tree = new JTree();
	private static TreeModel treeModel;
	
	static {
	}
	
	/**
	 * Method viewTemplate.
	 * @param tmplt :
	 */
	public static void viewTemplate(TemplateDescription tmplt) {
		tree.setCellRenderer(new KilimTemplateTreeCellRenderer());		
		Container cntnr = view_frame.getContentPane();
		cntnr.setLayout(new BorderLayout());
		JScrollPane scrllpn = new JScrollPane(tree);
		cntnr.add(BorderLayout.CENTER, scrllpn);
		view_frame.setSize(400, 400);
		
		ToolTipManager.sharedInstance().registerComponent(tree);
		
		String tmplt_nm = "error";
		tmplt_nm = tmplt.getName();
		
		view_frame.setTitle(WINDOW_NAME + " - " + tmplt_nm);
		
		treeModel = new KilimTemplateTreeModel(tmplt);
		tree.setModel(treeModel);
		tree.repaint();
		view_frame.setVisible(true);

		synchronized (action_listener) {
			try {
				action_listener.wait();
			} catch (InterruptedException exp) {
				exp.printStackTrace();
			}
		}
	}	

	/**
	 * Method updateView.
	 */
	public static void updateView() {
		tree.setModel(null);
		tree.setModel(treeModel);		
		view_frame.validate();
		view_frame.repaint();

		synchronized (action_listener) {
			try {
				action_listener.wait();
			} catch (InterruptedException exp) {
				exp.printStackTrace();
			}
		}
	}
}

class KilimTemplateTreeCellRenderer extends DefaultTreeCellRenderer {

	private static final String[] ICON_NAMES = { "provider.gif" , "transformer.gif" , "template_instance.gif" ,
												 "binding.gif" , "port.gif" , "template.gif" , "property.gif" ,
												 "trigger.gif" , "multiPort.gif" , "optionalPort.gif" , "slot.gif" ,
												 "plug.gif"};
												 
	private static final Icon[] ICONS = new Icon[ICON_NAMES.length];
	
	static {
		for (int i = 0; i < ICON_NAMES.length; i++) {
			ICONS[i] = getIcon(ICON_NAMES[i]);
		}
	}

	/**
	 * @see javax.swing.tree.TreeCellRenderer#getTreeCellRendererComponent(JTree, Object, boolean, boolean, boolean, int, boolean)
	 */
	public java.awt.Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded,
															boolean leaf, int row, boolean hasFocus) {
																
		super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
		
		if (value instanceof NamedElement) {
			String nm = ((NamedElement) value).getLocalName();
			setText(nm);
			
		} else if (value instanceof Binding) {
			if (ICONS[3] != null) {
				setIcon(ICONS[3]);
			}
			
			Binding bndng = (Binding) value;
			Iterator prvdrs = bndng.getBoundProviders();
			StringBuffer prvdrs_txt_bffr = new StringBuffer();
			boolean frst = true;
			for (; prvdrs.hasNext();) {
				Object nxt = prvdrs.next();
				if (frst) {
					frst = false;
				} else {
					prvdrs_txt_bffr.append(",");
				}
				if (nxt instanceof NamedElement) {
					prvdrs_txt_bffr.append(((NamedElement) nxt).getLocalName());
				} else if (nxt instanceof TpMethod) {
					TpMethod mthd = (TpMethod) nxt;
					prvdrs_txt_bffr.append(mthd.getSupport());
					prvdrs_txt_bffr.append(".");
					prvdrs_txt_bffr.append(mthd.getMethodName());
					prvdrs_txt_bffr.append("()");
				} else {
					prvdrs_txt_bffr.append(nxt);
				}
			}
			setText("Binding: {" + bndng.getPortName() + "} <-- " + prvdrs_txt_bffr.toString());
			
		}  else if (value instanceof Trigger) {
			
			if (ICONS[7] != null) {
				setIcon(ICONS[7]);
			}
			Trigger trggr = (Trigger) value;
			int evt_knd = trggr.getEventKind();
			String src_nm = trggr.getSourceName();
			Iterator trnsfrmrs = trggr.getTransformers();
			
			StringBuffer prvdrs_txt_bffr = new StringBuffer();
			boolean frst = true;
			for (; trnsfrmrs.hasNext();) {
				Object nxt = trnsfrmrs.next();
				if (frst) {
					frst = false;
				} else {
					prvdrs_txt_bffr.append(",");
				}
				if (nxt instanceof NamedElement) {
					prvdrs_txt_bffr.append(((NamedElement) nxt).getLocalName());
				} else if (nxt instanceof TpMethod) {
					TpMethod mthd = (TpMethod) nxt;
					prvdrs_txt_bffr.append(mthd.getSupport());
					prvdrs_txt_bffr.append(".");
					prvdrs_txt_bffr.append(mthd.getMethodName());
					prvdrs_txt_bffr.append("()");
				} else {
					prvdrs_txt_bffr.append(nxt);
				}
			}
			setText("Trigger on " + Trigger.EVENT_TYPE_NAME[evt_knd] + ": {" + src_nm + "} ~~> " + prvdrs_txt_bffr.toString());
		}
		
		if (value instanceof Provider) {
			
			if (ICONS[0] != null) {
				setIcon(ICONS[0]);
			}	
			
		} else if (value instanceof Transformer) {
			
			if (ICONS[1] != null) {
				setIcon(ICONS[1]);
			}
			
		} else if (value instanceof Instance) {
			
			if (ICONS[2] != null) {
				setIcon(ICONS[2]);
			}
			Instance instnc = (Instance) value;
			TemplateDescription tmplt = instnc.getTemplate();
			String tmplt_nm = getTemplateSuperAndSuperSuperTemplateName(tmplt);
			setText(instnc.getLocalName() + " [" + tmplt_nm + "]");
				
		} else if (value instanceof Port) {
			
			Port prt = (Port) value;
			Arity art = prt.getArity();
			
			if (art == Arity.COLLECTION) {
				if (ICONS[8] != null) {
					setIcon(ICONS[8]);
				}
			} else if (art == Arity.REQUIRED) {
				if (ICONS[4] != null) {
					setIcon(ICONS[4]);
				}
			} else {
				if (ICONS[9] != null) {
					setIcon(ICONS[9]);
				}
			}
			
			setText(prt.getLocalName() + " [" + (prt.isOffered() ? "offered" : "required") + "]");
			
		} else if (value instanceof TemplateDescription) {
			
			if (ICONS[5] != null) {
				setIcon(ICONS[5]);
			}
			TemplateDescription tmplt = (TemplateDescription) value;
			String tmplt_nm = getTemplateSuperAndSuperSuperTemplateName(tmplt);
			setText(tmplt_nm);			
				
		} else if (value instanceof Property) {
			
			Property prprt = (Property) value;
			setText(prprt.getLocalName() + " [" + prprt.getType() + "]: " + prprt.getValue());
			if (ICONS[6] != null) {
				setIcon(ICONS[6]);
			}
			
		} else if (value instanceof Slot) {
			
			Slot slt = (Slot) value;
			if (ICONS[10] != null) {
				setIcon(ICONS[10]);
			}
			
		} else if (value instanceof Plug) {
			
			Plug slt = (Plug) value;
			setText("Plugging [" + slt.getInstanceName() + "] in slot [" + slt.getSlotName() + "]");
			if (ICONS[11] != null) {
				setIcon(ICONS[11]);
			}
			
		}
		
		if (value instanceof TemplateElementImpl) {
			TemplateElementImpl elmnt = (TemplateElementImpl) value;
			Iterator dfntns = elmnt.getTemplateDefHierarchy();
			StringBuffer bffr = new StringBuffer("<html><body>Defined in ");
			boolean first = true;
			for (; dfntns.hasNext() ;) {
				if (first) {
					first = false;
				} else {
				 bffr.append(" -> ");
				}
				TemplateDescription tmplt_dsc = (TemplateDescription) dfntns.next();
				bffr.append("<b>");
				bffr.append(tmplt_dsc.getName());
				bffr.append("</b>");
			}
			bffr.append("</body></html>");
			setToolTipText(bffr.toString());
		}
		
		return this;
	}

	private static ImageIcon getIcon(String name) {
		URL url = ClassLoader.getSystemClassLoader().getSystemResource(name);
		if (url != null) {
			Image img = Toolkit.getDefaultToolkit().getImage(url);
			if (img != null) {
				return new ImageIcon(img);
			} else {
				return null;
			}
		} else {
			return null;
		}
	}
	
	private static final String getTemplateSuperAndSuperSuperTemplateName(TemplateDescription tmplt) {
		if (tmplt == null) {
			return "none";
		}
		
		StringBuffer bffr = new StringBuffer();
		bffr.append(tmplt.getName());
			
		TemplateDescription spr_spr_tmplt = null;
		TemplateDescription spr_tmpl = null;
		
		if (tmplt != null) {
			spr_tmpl = tmplt.getSuperTemplate();
		}
		if (spr_tmpl != null) {
			bffr.append(" -> ");
			bffr.append(spr_tmpl.getName());
			spr_spr_tmplt = spr_tmpl.getSuperTemplate();
			if (spr_spr_tmplt != null) {
				bffr.append(" -> ");
				bffr.append(spr_spr_tmplt.getName());
			}
		}
		return bffr.toString();
	}
}
