package org.objectweb.kilim.tools.jmx;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.DynamicMBean;
import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.InvalidAttributeValueException;
import javax.management.JMRuntimeException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanConstructorInfo;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;
import javax.management.MBeanRegistration;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import javax.management.ReflectionException;


import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.Property;
import org.objectweb.kilim.description.Slot;
import org.objectweb.kilim.description.TemplateElementImpl;
import org.objectweb.kilim.helpers.KilimHelper;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentInterface;
import org.objectweb.kilim.model.ComponentProperty;
import org.objectweb.kilim.model.ComponentSlot;
import org.objectweb.kilim.model.RtCollectionPort;
import org.objectweb.kilim.model.RtComponentInterface;
import org.objectweb.kilim.model.RtComponentProperty;
import org.objectweb.kilim.model.RtComponentSlot;
import org.objectweb.kilim.model.RtSingleValuePort;
import org.omg.CORBA.INTF_REPOS;

/**
 * @author delpiano
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class ComponentMBean implements DynamicMBean, MBeanRegistration {
	
	private static final String JMX_META_DATA_NAME = "JMX";

	private static MBeanServer mbeanserver;
	
	private MBeanInfo dMBeanInfo = null;
	private Component component;
	private String root_component_name;
	private String root_domain_name;

	public ComponentMBean(String template_name) {
		super();
		try {
			component = KilimHelper.newComponent(template_name , this.getClass());
		} catch (Exception e) {
			e.printStackTrace();
			throw new JMRuntimeException(e.getMessage());
		}
		buildDynamicMBeanInfo();
	}

	public ComponentMBean(Component component) {
		super();
		this.component = component;
		buildDynamicMBeanInfo();
	}
	
	private ComponentMBean(Component component , String root_component_name , String root_domain_name) {
		this(component);
		this.root_component_name = root_component_name;
		this.root_domain_name = root_domain_name;
	}
	
	public static void setDefaultMBeanServer(MBeanServer mbeanserver) {
		ComponentMBean.mbeanserver = mbeanserver;
	}

	/**
	 * @see javax.management.DynamicMBean#getAttribute(String)
	 */
	public Object getAttribute(String name)
		throws AttributeNotFoundException, MBeanException, ReflectionException {
			
		if (name.equals("InterfacesNames")) { return getPortsNames(); }
		if (name.equals("SlotsNames")) { return getSlotsNames(); }	
		
		ComponentInterface rslt = null;
		
		try {
			rslt = component.getInterface(denormalize(name));
		} catch (KilimException e) {
			throw new MBeanException(e);
		}
		
		if (rslt == null) { throw new AttributeNotFoundException(); }
		
		Object value = null;
		try {
			value = rslt.getValue();
		} catch (KilimException e) {
			throw new MBeanException(e);
		}
		
		return value;
	}
	
	private String cached_ports_names;
	
	private Object getPortsNames() {
	
		if (cached_ports_names != null) { return cached_ports_names; }
	
		StringBuffer prtsnms_bffr = new StringBuffer();
		boolean first = true;
		
		for (Iterator it = component.getInterfaces() ; it.hasNext() ;) {
			ComponentInterface intf = (ComponentInterface) it.next();
			if (intf.isSingleValuePort()) {
				if (first) { first = false; } else { prtsnms_bffr.append(" / "); }
				RtSingleValuePort port = (RtSingleValuePort) intf;
				prtsnms_bffr.append(port.getLocalName());
			}
		}
		
		cached_ports_names = prtsnms_bffr.toString();
		
		return cached_ports_names;
	
	}
	
	private String cached_slots_names;
	
	private Object getSlotsNames() {
		
		if (cached_slots_names != null) { return cached_slots_names; }
		
		StringBuffer slots_nms_bffr = new StringBuffer();
		boolean first = true;
		
		for (Iterator it = component.getSlots() ; it.hasNext() ;) {
			RtComponentSlot slt = (RtComponentSlot) it.next();
			if (first) { first = false; } else { slots_nms_bffr.append(" / "); }
			slots_nms_bffr.append(slt.getLocalName());
		}
		
		cached_slots_names = slots_nms_bffr.toString();
		
		return cached_slots_names;
		
	}

	/**
	 * @see javax.management.DynamicMBean#getAttributes(String[])
	 */
	public AttributeList getAttributes(String[] names) {
		
		AttributeList rslt = new AttributeList();
		
		for (int i = 0 ; i < names.length ; i++) {

			try  {
				rslt.add(getAttribute(names[i]));
			} catch (Exception e) {
				e.printStackTrace();
				continue;
			}
			
		}
		return rslt;
	}

	/**
	 * @see javax.management.DynamicMBean#getMBeanInfo()
	 */
	public MBeanInfo getMBeanInfo() {
		return dMBeanInfo;
	}

	/**
	 * @see javax.management.DynamicMBean#invoke(String, Object[], String[])
	 */
	public Object invoke(String method_name, Object[] params, String[] signature)
		throws MBeanException, ReflectionException {
		
		Class[] clsss = new Class[signature.length];
		for (int i = 0 ; i < signature.length ; i++) {
			try {
				clsss[i] = this.getClass().getClassLoader().loadClass(signature[i]);
			} catch (ClassNotFoundException e) {
				throw new ReflectionException(e);
			}
		}	
		
		Method method = null;
		try {
			method = this.getClass().getMethod(method_name , clsss);
		} catch (NoSuchMethodException e) {
			throw new ReflectionException(e);
		}
		
		Object result = null;
		try {
			result = method.invoke(this , params);
		} catch (InvocationTargetException e){
			throw new ReflectionException(e);
		} catch (IllegalAccessException e){
			throw new ReflectionException(e);
		}
		
		return result;

	}

	/**
	 * @see javax.management.DynamicMBean#setAttribute(Attribute)
	 */
	public void setAttribute(Attribute att)
		throws
			AttributeNotFoundException,
			InvalidAttributeValueException,
			MBeanException,
			ReflectionException {
				
		String nm = denormalize(att.getName());
		Object vl = att.getValue();
		
		ComponentInterface intf = null;
		try {
			intf = component.getInterface(nm);
		} catch (KilimException e) {
			throw new MBeanException(e);
		}
		
		if (intf == null) { throw new AttributeNotFoundException(nm);}
		
		if (!(intf instanceof RtComponentProperty)) {
			throw new AttributeNotFoundException(nm);
		}
		
		RtComponentProperty prpt = (RtComponentProperty) intf;
		
		try {
			prpt.setValue(vl);
		} catch (KilimException e) {
			throw new InvalidAttributeValueException(vl.toString() + "[" + vl.getClass().toString() + "]");
		}
		
		//Quelle interface unbind/getValuer ???
		
		
	}

	/**
	 * @see javax.management.DynamicMBean#setAttributes(AttributeList)
	 */
	public AttributeList setAttributes(AttributeList attlist) {
		
		AttributeList rslt = new AttributeList();
		
		for (Iterator it = attlist.iterator() ; it.hasNext() ; ) {
			Attribute crrnt_att = (Attribute) it.next();
			
			try {
				setAttribute(crrnt_att);
			} catch (Exception e) {
				e.printStackTrace();
				continue;
			} 
			
			rslt.add(crrnt_att);
			
		}
		
		return rslt;

	}
	
	//Specific monitoring methods -----------------------------------------
	
	public String getValueOnInterface(String name) {
		
		ComponentInterface intrfc = null;
		try {
			
			intrfc = component.getInterface(name);

			if (intrfc != null) {
				return intrfc.getValue().toString();
			}

		} catch (KilimException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	public boolean rebindInterface(String name) {
		
		RtSingleValuePort intrfc = null;
		
		try {
			
			ComponentInterface retrieved = component.getInterface(name);
			 
			if (!(retrieved instanceof RtSingleValuePort)) {
				return false;
			}
			
			intrfc = (RtSingleValuePort) retrieved;

			if (intrfc != null) {
				intrfc.update();
				return true;
			}

		} catch (KilimException e) {
			e.printStackTrace();
		}
		
		return false;
	}
	
	public Component getComponent() {
		return component;
	}
	
	public boolean fork() {
		
		Component forked_component = null;
		try {
			forked_component = component.fork();
		} catch (KilimException e) {
			e.printStackTrace();
			return false;
		}
		
		
		ComponentMBean forked_component_mbean = new ComponentMBean(forked_component);
		
		try {
			mbeanserver.registerMBean(forked_component_mbean , null);
		} catch (MBeanRegistrationException e) {
			e.printStackTrace();
			return false;			
		} catch (InstanceAlreadyExistsException e) {
			e.printStackTrace();
			return false;
		} catch (NotCompliantMBeanException e) {
			e.printStackTrace();
			return false;
		}
			
		return true;
	}
	
	public boolean plugComponent(String slot_nm , ObjectName obj_nm) {
		return plugAndUnplugComponent(true , slot_nm , obj_nm);
	}
	
	public boolean unplugComponent(String slot_nm , ObjectName obj_nm) {
		return plugAndUnplugComponent(false , slot_nm , obj_nm);
	}
	
	public boolean plugAndUnplugComponent(boolean is_plug , String slot_nm , ObjectName obj_nm) {
		
		System.out.println("Plugging  \"" + obj_nm + "\" in slot \"" + slot_nm + "\".");
		
		 Component plug;
		 try {
		 	plug = (Component) mbeanserver.invoke(obj_nm , "getComponent" , new Object[0] , new String[0]);
		 } catch (InstanceNotFoundException e) {
		 	e.printStackTrace();
		 	return false;
		 } catch (MBeanException e) {
		 	e.printStackTrace();
		 	return false;
		 } catch (ReflectionException e) {
		 	e.printStackTrace();
		 	return false;
		 }
		 
		 ComponentSlot slt = null;
		 try {
		 	slt = component.getSlot(slot_nm);
		 } catch (KilimException e) {
		 	e.printStackTrace();
		 	return false;
		 }
		 
		 try {
		 	 if (is_plug) {
				slt.plug(plug);
		 	 } else {
		 	 	slt.unplug(plug);
		 	 }
		 } catch (KilimException e) {
		 	e.printStackTrace();
		 	return false;
		 }
		 
		 return true;
	}
	
	/*
     * -----------------------------------------------------
     * PRIVATE METHODS
     * -----------------------------------------------------
     */
  
    /**
     * Build the private dMBeanInfo field,
     * which represents the management interface exposed by the MBean;
     * that is, the set of attributes, constructors, operations and notifications
     * which are available for management. 
     *
     * A reference to the dMBeanInfo object is returned by the getMBeanInfo() method
     * of the DynamicMBean interface. Note that, once constructed, an MBeanInfo object is immutable.
     */
    private void buildDynamicMBeanInfo() {

	
	LinkedList prprts = new LinkedList();

	Iterator edtbl_prprts = getEditableProperties();
	Iterator mntrd_intrfcs = getMonitoredInterfaces();
	
	LinkedList attributes_list = new LinkedList();
	
	attributes_list.add(new MBeanAttributeInfo("InterfacesNames" ,
						"java.lang.String",
						"Ports Names",
						true,
						false,
						false)
						);
						
	attributes_list.add(new MBeanAttributeInfo("SlotsNames" ,
						"java.lang.String",
						"Slots Names",
						true,
						false,
						false)
						);
						
	if (mntrd_intrfcs != null) {
		for (; mntrd_intrfcs.hasNext() ;) {
				ComponentInterface crrnt = (ComponentInterface) mntrd_intrfcs.next();
				Class clss = String.class;
				try {
					clss = crrnt.getValue().getClass();
				} catch (KilimException exc) {}
				attributes_list.add(new MBeanAttributeInfo(normalize(crrnt.getLocalName()) ,
									clss.getName(),
									crrnt.getQualifiedName(),
									true,
									false,
									false)
									);
		}
	}
	
	if (edtbl_prprts != null) {
		for (; edtbl_prprts.hasNext() ;) {
				ComponentProperty crrnt = (ComponentProperty) edtbl_prprts.next();
				Property property = (Property) crrnt.getElementDescription();
				Class type = property.getType();
				attributes_list.add(new MBeanAttributeInfo(normalize(crrnt.getLocalName()) ,
									type.getName(),
									crrnt.getQualifiedName(),
									true,
									true,
									false)
									);
		}
	}

	//Registering all public properties of the component as attributes

	MBeanAttributeInfo[] dAttributes = new MBeanAttributeInfo[attributes_list.size()];
	attributes_list.toArray(dAttributes);	
        
	Constructor[] constructors = this.getClass().getConstructors();
	
	MBeanConstructorInfo[] dConstructors = new MBeanConstructorInfo[1];
	dConstructors[0] = new MBeanConstructorInfo("ComponentMBean(): Constructs a Component",
						    constructors[0]);
    
    
    MBeanOperationInfo[] dOperations = new MBeanOperationInfo[6];
    
    MBeanParameterInfo[] parameters_0 = new MBeanParameterInfo[1];
	parameters_0[0] = new MBeanParameterInfo("Interface" , "java.lang.String" , "The name of the Interface to get the Value from."); 
   	dOperations[0] = new MBeanOperationInfo("getValueOnInterface" , "Get the value of a given Interface, bind it if necessary." , parameters_0 , "java.lang.String" , MBeanOperationInfo.ACTION);

	dOperations[1] = new MBeanOperationInfo("rebindInterface" , "Updates the state of a given Interface's Value." , parameters_0 , "java.lang.String" , MBeanOperationInfo.ACTION);
	
	Method meth = null;
    try {
    	meth = this.getClass().getMethod("getComponent" , null);
    } catch (NoSuchMethodException e) {
    	e.printStackTrace();
    } 
	dOperations[2] = new MBeanOperationInfo("Get Component" ,meth);

	meth = null;
    try {
    	meth = this.getClass().getMethod("fork" , null);
    } catch (NoSuchMethodException e) {
    	e.printStackTrace();
    } 
	dOperations[3] = new MBeanOperationInfo("Fork Component" ,meth);
	
	MBeanParameterInfo[] parameters = new MBeanParameterInfo[2];
	parameters[0] = new MBeanParameterInfo("Slot" , "java.lang.String" , "The Slot where the Component will be Plugged.");
	parameters[1] = new MBeanParameterInfo("Component" , "javax.management.ObjectName" , "The Name of the Component to Plug.");
	dOperations[4] = new MBeanOperationInfo("plugComponent" , "Plug Component into given Slot" , parameters , "java.lang.Boolean" , MBeanOperationInfo.ACTION);
	
	
	parameters[0] = new MBeanParameterInfo("Slot" , "java.lang.String" , "The Slot from which the Component will be Unplugged.");
	parameters[1] = new MBeanParameterInfo("Component" , "javax.management.ObjectName" , "The Name of the Component to Unplug.");
	dOperations[5] = new MBeanOperationInfo("unplugComponent" , "Unplug Component from given Slot" , parameters , "java.lang.Boolean" , MBeanOperationInfo.ACTION);

        
	dMBeanInfo = new MBeanInfo(this.getClass().getName(),
				   "JMX Supervision of Kilim Components",
				   dAttributes,
				   dConstructors,
				   dOperations,
				   new MBeanNotificationInfo[0]);
    }
    
    private static final String normalize(String nm) {
    	return nm.replaceAll(" ","__");
    }
    
        private static final String denormalize(String nm) {
    	return nm.replaceAll("__"," ");
    }
    
    private Component getJMXDataContainer() {
    	return component.getSubComponent(JMX_META_DATA_NAME);
    }
    
    private Iterator getMetaData(String well_known_name) {
    	
    	Component jmx_cntnr = getJMXDataContainer();
		if (jmx_cntnr == null) { return null; }
	
		ComponentInterface intrfc = null;
		try {
		 intrfc = jmx_cntnr.getInterface(well_known_name);
		} catch (KilimException e) {
			e.printStackTrace();
		}
		
		if (intrfc == null) { return null; }
		if (!(intrfc instanceof RtCollectionPort)) { return null; }
		
		Iterator rslt = ((RtCollectionPort) intrfc).getBoundProviders();
		
    	return rslt;
    }
    
    private Iterator getMonitoredInterfaces() { return getMetaData("visible"); }    
    private Iterator getEditableProperties() { return getMetaData("editable"); }
       
    private Iterator getMonitoredSubComponents() {
    	
    	Iterator cmpnnts = getMetaData("monitored sub-components");
    	if (cmpnnts == null) return null;
		
		LinkedList result = new LinkedList();
		for (; cmpnnts.hasNext() ;) {
			Object cmpnnt = null;
			try {
				ComponentInterface intrfc = (ComponentInterface) cmpnnts.next();
				cmpnnt = intrfc.getValue();
				if (cmpnnt.equals("All")) {
					Iterator sb_cmpnnts = component.getSubComponents();
					return sb_cmpnnts;
				}
			} catch (KilimException e) { e.printStackTrace(); }
			result.add(cmpnnt);
		}
		
    	return result.iterator();
    }

	// --------- javax.management.MBeanRegistration Interface Implementation -------------------------------------

	/**
	 * @see javax.management.MBeanRegistration#postDeregister()
	 */
	public void postDeregister() {
	}

	/**
	 * @see javax.management.MBeanRegistration#postRegister(Boolean)
	 */
	public void postRegister(Boolean arg0) {
		
		Iterator it = getMonitoredSubComponents();
		if (it == null) { return; }
		
		for (; it.hasNext() ;) {
			try {
				Component nxt = (Component) it.next();
				if (nxt.getLocalName().equals(JMX_META_DATA_NAME)) { continue; }
				ComponentMBean mbn = new ComponentMBean(nxt , root_component_name , root_domain_name);
				mbeanserver.registerMBean(mbn , null);
			} catch (NotCompliantMBeanException e) {
				e.printStackTrace();
				continue;
			} catch (MBeanRegistrationException e) {
				e.printStackTrace();
				continue;
			} catch (InstanceAlreadyExistsException e) {
				e.printStackTrace();
				continue;
			}
		}
	}

	/**
	 * @see javax.management.MBeanRegistration#preDeregister()
	 */
	public void preDeregister() throws Exception {
	}

	/**
	 * @see javax.management.MBeanRegistration#preRegister(MBeanServer, ObjectName)
	 */
	public ObjectName preRegister(MBeanServer srvr, ObjectName obj_nm)
		throws Exception {
		
		ObjectName trying_with_nm = null;
		
		if (root_component_name == null) {
			
			//This is a root component.
			
			//Guarantee that the root name is unique
			
			if (obj_nm == null) {
				obj_nm = new ObjectName("KilimComponent:name=FORKED");
			}
				
			trying_with_nm = obj_nm;
		
			ObjectInstance obj_instnc = null;
			try {
				obj_instnc = srvr.getObjectInstance(trying_with_nm);
			} catch (InstanceNotFoundException e) {}
	
			while (obj_instnc != null) {
				Hashtable nms = trying_with_nm.getKeyPropertyList();
				String new_nm = (String) nms.get("name");
				new_nm = new_nm + "_c";
				nms.put("name" , new_nm);
				trying_with_nm = new ObjectName(trying_with_nm.getDomain() , nms);
				
				try {
					obj_instnc = srvr.getObjectInstance(trying_with_nm);
				} catch (InstanceNotFoundException e) {
					obj_instnc = null;
				}
			}	
		
				root_component_name = trying_with_nm.getKeyProperty("name");
				root_domain_name = trying_with_nm.getDomain();

			
			return trying_with_nm;
		}
		
		//This is a sub-component
		
		String nm = root_domain_name + ":name=" + root_component_name + "/" + component.getQualifiedName();
		nm = nm.replaceAll(" " , "_");
		trying_with_nm = new ObjectName(nm);

		return trying_with_nm;

	}

}
