/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.mapping;

import org.objectweb.kilim.KilimException;

/**
 * This mapper does not perform any action. it is just an indirection to the next mapper, when piped to another mapper.
 * @author horn
 */
public class NullMapper implements Mapper {
	private Mapper wrappedMapper;
   
	/**
	 * Method JavaLogMapper.
	 * @param aMapper :
	 */
    public NullMapper(Mapper aMapper) {
    	wrappedMapper = aMapper;
    }

	/**
	 * @see org.objectweb.kilim.model.mapping.Mapper#enterContext(Object)
	 */
	public void enterContext(MappingContext aContext) throws KilimException { }
	
	/**
	 * @see org.objectweb.kilim.model.mapping.Mapper#leaveContext()
	 */
	public void leaveContext(MappingContext aContext) throws KilimException { }

	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getGetterValue(Object, boolean, String, MappingContext)
	 */
	public Object getGetterValue (Object aSupport, boolean isStatic, String fieldName, MappingContext aContext) throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getGetterValue (aSupport, isStatic, fieldName, aContext);
		}
		
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#executeSetter(Object, boolean, String, Object, MappingContext)
	 */
	public void executeSetter(Object aSupport, boolean isStatic, String fieldName, Object toBeSet, MappingContext aContext) throws KilimException {
		if (wrappedMapper != null) {
			wrappedMapper.executeSetter(aSupport, isStatic, fieldName, toBeSet, aContext);
		}
	}

	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getMethodValue(Object, boolean, String, Object[], String[], MappingContext)
	 */
	public Object getMethodValue(Object aSupport, boolean isStatic, String aMethodName, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getMethodValue (aSupport, isStatic, aMethodName, paramObjects, typeNames, aContext);
		}
		
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#executeMethod(Object, boolean, String, Object[], String[], MappingContext)
	 */
	public void executeMethod(Object aSupport, boolean isStatic, String aMethodName, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {
		getMethodValue(aSupport, isStatic, aMethodName, paramObjects, typeNames, aContext);
	}	
			
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getConstructorValue(Class, Object[], String[], MappingContext)
	 */
	public Object getConstructorValue(Class aClass, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {		
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getConstructorValue(aClass, paramObjects, typeNames, aContext);
		}
				
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#executeConstructor(Class, Object[], String[], MappingContext)
	 */
	public void executeConstructor(Class aClass, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {
		getConstructorValue(aClass, paramObjects, typeNames, aContext);
	}			
			

	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getExternalValue(Object, MappingContext)
	 */
	public Object getExternalValue(Object aValue, MappingContext aContext)  throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getExternalValue(aValue, aContext);
		}
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getPropertyValue(Object, MappingContext)
	 */
	public Object getPropertyValue(Object aValue, MappingContext aContext)  throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getPropertyValue(aValue, aContext);
		}
		return resultValue;
	}
		
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getClassValue(String, MappingContext)
	 */
	public Object getClassValue(String aClassName, MappingContext aContext) throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getClassValue(aClassName, aContext);
		}
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getEventSourceValue(MappingContext)
	 */
	public Object getEventSourceValue(MappingContext aContext) throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getEventSourceValue(aContext);
		}
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getNullElementValue(MappingContext)
	 */
	public Object getNullElementValue(MappingContext aContext)  throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getNullElementValue(aContext);
		}
		return resultValue;

	}

	/**
	 * @see org.objectweb.kilim.model.mapping.Mapper#executeNullElement(MappingContext)
	 */
	public void executeNullElement(MappingContext aContext)  throws KilimException {
		if (wrappedMapper != null) {
			wrappedMapper.executeNullElement(aContext);
		}
	}
}