/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.mapping;

import java.util.HashMap;
import javax.swing.JTextArea;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
public class TextAreaMapper implements Mapper {
	private Mapper wrappedMapper;
	private JTextArea textArea;
	private String prefix = "\n    ";
	
	/**
	 * Method JavaLogMapper.
	 * @param aMapper :
	 */
    public TextAreaMapper(Mapper aMapper) {
    	wrappedMapper = aMapper;
    	textArea = new JTextArea("++++++++++++++START TRACE++++++++++++++++++++");
    }
    
	/**
	 * @see java.lang.Object#Object()
	 */
    public TextAreaMapper() {
		this(null);
    }
    
	/**
	 * Method getTreeModel.
	 * @return KilimTraceTreeModel
	 */
    public JTextArea getTextArea() {
    	return textArea;
    }
    
    /**
	 * @see org.objectweb.kilim.model.mapping.Mapper#enterContext(Object)
	 */
	public void enterContext(MappingContext aContext) throws KilimException { }
	
	/**
	 * @see org.objectweb.kilim.model.mapping.Mapper#leaveContext()
	 */
	public void leaveContext(MappingContext aContext) throws KilimException { }

	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getGetterValue(Object, boolean, String, MappingContext)
	 */
	public Object getGetterValue (Object aSupport, boolean isStatic, String fieldName, MappingContext aContext) throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getGetterValue (aSupport, isStatic, fieldName, aContext);
		}
      	textArea.append(prefix + getResultString(resultValue) + ((isStatic) ? "[static] " : "") + normalizeToString(aSupport) + "." + fieldName + ";");
      	//prefix = "\n";	
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#executeSetter(Object, boolean, String, Object, MappingContext)
	 */
	public void executeSetter(Object aSupport, boolean isStatic, String fieldName, Object toBeSet, MappingContext aContext) throws KilimException {
		textArea.append(prefix + ((isStatic) ? "[static] " : "") + normalizeToString(aSupport) + "." + fieldName + " = " + toBeSet + ";");
		
		if (wrappedMapper != null) {
			wrappedMapper.executeSetter(aSupport, isStatic, fieldName, toBeSet, aContext);
		}
		//prefix = "\n";	
	}

	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getMethodValue(Object, boolean, String, Object[], String[], MappingContext)
	 */
	public Object getMethodValue(Object aSupport, boolean isStatic, String aMethodName, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getMethodValue (aSupport, isStatic, aMethodName, paramObjects, typeNames, aContext);
		}
		
      	StringBuffer bffr = new StringBuffer(getResultString(resultValue) + ((isStatic) ? "[static] " : "") + normalizeToString(aSupport) + "." + aMethodName);
		addParameters(bffr , paramObjects , typeNames);
		bffr.append(";");
		textArea.append(prefix + bffr.toString());
		//prefix = "\n";	
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#executeMethod(Object, boolean, String, Object[], String[], MappingContext)
	 */
	public void executeMethod(Object aSupport, boolean isStatic, String aMethodName, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {	   
		if (wrappedMapper != null) {
			wrappedMapper.executeMethod(aSupport, isStatic, aMethodName, paramObjects, typeNames, aContext);
		}
		
      	StringBuffer bffr = new StringBuffer(((isStatic) ? "[static] " : "") + normalizeToString(aSupport) + "." + aMethodName);
		addParameters(bffr , paramObjects , typeNames);
		bffr.append(";");
		textArea.append(prefix + bffr.toString());
		//prefix = "\n";	
	}	
			
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getConstructorValue(Class, Object[], String[], MappingContext)
	 */
	public Object getConstructorValue(Class aClass, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {		
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getConstructorValue(aClass, paramObjects, typeNames, aContext);
		}
		
      	StringBuffer bffr = new StringBuffer(getVariableName(resultValue) + " = new " + aClass.getName());
		addParameters(bffr , paramObjects , typeNames);
		bffr.append(";");
		textArea.append(prefix + bffr.toString());
		//prefix = "\n";	
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#executeConstructor(Class, Object[], String[], MappingContext)
	 */
	public void executeConstructor(Class aClass, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {		   
		if (wrappedMapper != null) {
			wrappedMapper.getConstructorValue(aClass, paramObjects, typeNames, aContext);
		}
		
      	StringBuffer bffr = new StringBuffer("new " + aClass.getName());
		addParameters(bffr , paramObjects , typeNames);
		bffr.append(";");
		textArea.append(prefix + bffr.toString());
		//prefix = "\n";	
	}			

	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getExternalValue(Object, MappingContext)
	 */
	public Object getExternalValue(Object aValue, MappingContext aContext)  throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getExternalValue(aValue, aContext);
		}
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getPropertyValue(Object, MappingContext)
	 */
	public Object getPropertyValue(Object aValue, MappingContext aContext)  throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getPropertyValue(aValue, aContext);
		}
		return resultValue;
	}
		
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getClassValue(String, MappingContext)
	 */
	public Object getClassValue(String aClassName, MappingContext aContext) throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getClassValue(aClassName, aContext);
		}
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getEventSourceValue(MappingContext)
	 */
	public Object getEventSourceValue(MappingContext aContext) throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getEventSourceValue(aContext);
		}
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getNullElementValue(MappingContext)
	 */
	public Object getNullElementValue(MappingContext aContext)  throws KilimException {
		Object resultValue = null;		   
		if (wrappedMapper != null) {
			resultValue = wrappedMapper.getNullElementValue(aContext);
		}
		return resultValue;

	}

	/**
	 * Method executeNullElement.
	 * @param aContext :
	 */
	public void executeNullElement(MappingContext aContext)  {
		if (wrappedMapper != null) {
			executeNullElement(aContext);
		}
	}
	
	//Logging facilities ------------------------------------------------------------------
	
	private static final void addParameters(StringBuffer bffr , Object[] paramObjects, String[] typeNames) {
			bffr.append("(");
			for (int i = 0 ; i < paramObjects.length ; i++) {
				if (i != 0) { bffr.append(" , "); }
				bffr.append(normalizeToString(paramObjects[i]));
			}
			bffr.append(")");
	}
	
	private static String normalizeToString(Object obj) {		
		if (obj instanceof Number) { return obj.toString(); }	
		return getVariableName(obj);
	}
	
	private static HashMap variables = new HashMap() , inverse_table = new HashMap();
	
	private static String getResultString(Object obj) {
		String rslt = getVariableName(obj);
		if (rslt == null) { return ""; }
		return rslt + " = ";
	}
	
	private static String getVariableName(Object obj) {
		
		if (obj == null) { return null; }
		
		Object vl = inverse_table.get(obj);
		if (vl != null) { return (String) vl; }
		
		String clss_nm = obj.getClass().getName();
		String nm = clss_nm.substring(clss_nm.lastIndexOf('.') + 1);
		nm = nm.toLowerCase();
		Object allrd_thr = variables.get(nm);
		int nb = 0;
		while (allrd_thr != null) {
			allrd_thr = variables.get(nm + (++nb));
		}
		if (nb != 0) { nm = nm + nb; }
		variables.put (nm , obj);
		inverse_table.put(obj , nm);		
		return nm;
	}
}